@@ -6,10 +6,10 @@
 //                  Service de Police Judiciaire
 //                  Section Nouvelles Technologies
 // ****************************************************************************
-//  Module:         Multithreaded AFF (AAFF = Avanced AFF)
+//  Module:         Multithreaded AFF (AAFF = Advanced AFF)
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -31,7 +31,7 @@
 
 #include <netinet/in.h>
 #include <string.h>
-#include <time.h>    //lint !e537 repeated include file
+#include <time.h>    //lint !e537 !e451 repeated include file
 #include <stdlib.h>
 #include <zlib.h>
 
@@ -78,8 +78,8 @@ typedef struct
    unsigned int NameLen;
    unsigned int DataLen;
    unsigned int Argument;          // Named "flags" in original aff source, named "arg" in afinfo output.
-   char         Name[];
-} t_AffSegmentHeader;
+   char         Name[];            //lint !e1501
+} __attribute__ ((packed)) t_AffSegmentHeader;
 
 // Between header and footer lie the segment name and the data
 
@@ -87,7 +87,7 @@ typedef struct
 {
    char         Magic[4];
    unsigned int SegmentLen;
-} t_AffSegmentFooter;
+} __attribute__ ((packed)) t_AffSegmentFooter;
 
 
 // ------------------
@@ -100,11 +100,16 @@ typedef struct _t_Aaff
    bool                 OpenedForWrite;
    unsigned long long   PagesWritten;
    unsigned long long   PagesRead;
+   t_HashContextMD5     HashContextMD5; // For image file MD5 calculation during verification
+   QString            *pFilename;
+
 
    t_AffSegmentHeader   SegmentHeader;  // Optimisation: This header and this footer only need to be
    t_AffSegmentFooter   SegmentFooter;  // allocated and initialised once, and can be used again and again
    unsigned char      *pUncompressBuffer;
    unsigned int         UncompressBufferLen;
+   unsigned long long   BytesVerified;
+   unsigned long long   DeviceSize;
 } t_Aaff;
 
 #define AAFF_MD5_LEN                16
@@ -120,33 +125,6 @@ unsigned char AaffBadSectorMarker[AAFF_BADSECTORMARKER_MAXLEN];
    if ((Fn) != 1)                         \
       CHK (ERROR_AAFF_CANNOT_WRITE_FILE)
 
-
-// -------------------
-//  Utility functions
-// -------------------
-
-static bool AaffIsZero (unsigned char *pData, unsigned int DataLen)
-{
-   long long          *pBuff    = (long long *)pData;   //lint !e826   Suspicious pointer-to-pointer conversion
-   const unsigned int   CmpSize = sizeof (long long);   // 64 bit operations for optimal performance on amd64 processors
-
-   while (DataLen >= CmpSize)
-   {
-      if (*pBuff++)
-        return false;
-      DataLen -= CmpSize;
-   }
-
-   pData = (unsigned char *) pBuff;
-   while (DataLen--)
-   {
-      if (*pData++)
-         return false;
-   }
-
-   return true;
-}
-
 // -------------------
 //  Segment functions
 // -------------------
@@ -226,22 +204,32 @@ APIRET AaffCopyBadSectorMarker (unsigned char *pBuffer, unsigned int Len)
 
 static APIRET AaffCreateHandle (t_pAaff *ppAaff)
 {
-   *ppAaff = (t_pAaff) UTIL_MEM_ALLOC(sizeof(t_Aaff));
-   if (*ppAaff == NULL)
+   t_pAaff pAaff;
+
+   pAaff = (t_pAaff) UTIL_MEM_ALLOC(sizeof(t_Aaff));
+   if (pAaff == NULL)
       CHK (ERROR_AAFF_MEMALLOC_FAILED)
 
-   memset (*ppAaff, 0, sizeof(t_Aaff));
+   memset (pAaff, 0, sizeof(t_Aaff));
+
+   pAaff->pFilename = new QString;
+
+   *ppAaff = pAaff;
 
    return NO_ERROR;
 }
 
 static APIRET AaffDestroyHandle (t_pAaff *ppAaff)
 {
-   if ((*ppAaff)->pUncompressBuffer)
-      free ((*ppAaff)->pUncompressBuffer);  // Must released with free, as realloc is used for allocating it
+   t_pAaff pAaff = *ppAaff;
 
-   memset (*ppAaff, 0, sizeof(t_Aaff));
-   UTIL_MEM_FREE (*ppAaff);
+   if (pAaff->pUncompressBuffer)
+      free (pAaff->pUncompressBuffer);  // Must released with free, as realloc is used for allocating it
+
+   delete pAaff->pFilename;
+
+   memset (pAaff, 0, sizeof(t_Aaff));
+   UTIL_MEM_FREE (pAaff);
    *ppAaff = NULL;
 
    return NO_ERROR;
@@ -262,6 +250,7 @@ APIRET AaffOpen (t_pAaff *ppAaff, const char *pFilename, unsigned long long Devi
 
    CHK (AaffCreateHandle (&pAaff))
 
+   *(pAaff->pFilename) = pFilename;
    pAaff->OpenedForWrite = true;
    pAaff->pFile = fopen64 (pFilename, "w");
    if (pAaff->pFile == NULL)
@@ -296,14 +285,14 @@ APIRET AaffOpen (t_pAaff *ppAaff, const char *pFilename, unsigned long long Devi
 }
 //lint -restore
 
-APIRET AaffClose (t_pAaff *ppAaff, unsigned long long BadSectors, const unsigned char *pMD5, const unsigned char *pSHA256, int Duration)
+APIRET AaffClose (t_pAaff *ppAaff, unsigned long long BadSectors, const unsigned char *pMD5, const unsigned char *pSHA256, uint Duration)
 {
    t_pAaff pAaff = *ppAaff;
 
-   CHK (AaffWriteSegmentU64 (pAaff, AFF_SEGNAME_BADSECTORS  , BadSectors))
-   CHK (AaffWriteSegment    (pAaff, AFF_SEGNAME_MD5         , 0, pMD5   , AAFF_MD5_LEN   ))
-   CHK (AaffWriteSegment    (pAaff, AFF_SEGNAME_SHA256      , 0, pSHA256, AAFF_SHA256_LEN))
-   CHK (AaffWriteSegmentArg (pAaff, AFF_SEGNAME_DURATION    , Duration))
+   CHK (AaffWriteSegmentU64 (pAaff, AFF_SEGNAME_BADSECTORS, BadSectors))
+   CHK (AaffWriteSegment    (pAaff, AFF_SEGNAME_MD5       , 0, pMD5   , AAFF_MD5_LEN   ))
+   CHK (AaffWriteSegment    (pAaff, AFF_SEGNAME_SHA256    , 0, pSHA256, AAFF_SHA256_LEN))
+   CHK (AaffWriteSegmentArg (pAaff, AFF_SEGNAME_DURATION  , Duration))
 
    if (fflush (pAaff->pFile))
    {
@@ -320,7 +309,7 @@ APIRET AaffClose (t_pAaff *ppAaff, unsigned long long BadSectors, const unsigned
 }
 
 //lint -save -esym(613,pPreProcess)   Possible use of null pointer
-APIRET AaffPreprocess (t_pAaffPreprocess *ppPreprocess, unsigned char *pDataIn, unsigned int DataLenIn, unsigned char *pDataOut, unsigned int DataLenOut)
+APIRET AaffPreprocess (t_pAaffPreprocess *ppPreprocess, unsigned char *pDataIn, unsigned int DataLenIn, unsigned char *pDataOut, uint DataLenOut)
 {
    t_pAaffPreprocess pPreProcess;
    int                rc;
@@ -336,7 +325,7 @@ APIRET AaffPreprocess (t_pAaffPreprocess *ppPreprocess, unsigned char *pDataIn,
 
    // Check if zero
    // -------------
-   pPreProcess->Zero = AaffIsZero (pDataIn, DataLenIn);
+   pPreProcess->Zero = UtilIsZero (pDataIn, DataLenIn);
    if (pPreProcess->Zero)
       return NO_ERROR;
 
@@ -382,7 +371,54 @@ APIRET AaffWrite (t_pAaff pAaff, t_pAaffPreprocess pPreprocess, const unsigned c
 //  API read functions
 // ---------------------
 
-APIRET AaffOpen (t_pAaff *ppAaff, const char *pFilename)
+static APIRET AaffReallocUncompressBuffer (t_pAaff pAaff, unsigned int NewLen)
+{
+   if (NewLen > pAaff->UncompressBufferLen)
+   {
+      pAaff->pUncompressBuffer = (unsigned char *) realloc (pAaff->pUncompressBuffer, NewLen);
+      if (pAaff->pUncompressBuffer == NULL)
+         return ERROR_AAFF_MEMALLOC_FAILED;
+      pAaff->UncompressBufferLen = NewLen;
+   }
+   return NO_ERROR;
+}
+
+static unsigned long long AaffGetCurrentSeekPos (t_Aaff *pAaff)
+{
+   return ftello64 (pAaff->pFile);
+}
+
+APIRET AaffGetImageFileSize  (t_Aaff *pAaff, unsigned long long *pSize)
+{
+   unsigned long long CurrentSeekPos;
+   int                rc;
+
+   CurrentSeekPos = AaffGetCurrentSeekPos (pAaff);
+   rc = fseeko64 (pAaff->pFile, 0, SEEK_END);
+   if (rc)
+      return ERROR_AAFF_CANNOT_SEEK;
+
+   *pSize = AaffGetCurrentSeekPos (pAaff);
+
+   rc = fseeko64 (pAaff->pFile, CurrentSeekPos, SEEK_SET);
+   if (rc)
+      return ERROR_AAFF_CANNOT_SEEK;
+
+   return NO_ERROR;
+}
+
+static APIRET AaffReadFile (t_Aaff *pAaff, void *pData, unsigned int DataLen)
+{
+   if (fread (pData, DataLen, 1, pAaff->pFile) != 1)
+      CHK (ERROR_AAFF_CANNOT_READ_DATA)
+
+   if (CONFIG(CalcImageFileMD5))
+      CHK (HashMD5Append (&pAaff->HashContextMD5, pData, DataLen))
+
+   return NO_ERROR;
+}
+
+APIRET AaffOpen (t_pAaff *ppAaff, const char *pFilename, unsigned long long DeviceSize)
 {
    t_pAaff pAaff;
    char     Signature[strlen(AFF_HEADER)+1];
@@ -394,6 +430,13 @@ APIRET AaffOpen (t_pAaff *ppAaff, const char *pFilename)
    pAaff->pUncompressBuffer   = NULL;
    pAaff->UncompressBufferLen = 0;
    pAaff->PagesRead           = 0;
+   pAaff->BytesVerified       = 0;
+   pAaff->DeviceSize          = DeviceSize;
+   *(pAaff->pFilename)        = pFilename;
+
+   if (CONFIG(CalcImageFileMD5))
+      CHK (HashMD5Init (&pAaff->HashContextMD5))
+
    pAaff->pFile = fopen64 (pFilename, "r");
    if (pAaff->pFile == NULL)
    {
@@ -403,9 +446,7 @@ APIRET AaffOpen (t_pAaff *ppAaff, const char *pFilename)
 
    // Check signature
    // ---------------
-   if (fread (&Signature, sizeof(Signature), 1, pAaff->pFile) != 1)
-      return ERROR_AAFF_CANNOT_READ_SIGNATURE;
-
+   CHK (AaffReadFile (pAaff, &Signature, sizeof(Signature)))
    if (memcmp (Signature, AFF_HEADER, sizeof(Signature)) != 0)
       return ERROR_AAFF_INVALID_SIGNATURE;
 
@@ -425,7 +466,7 @@ APIRET AaffClose (t_pAaff *ppAaff)
 }
 
 
-APIRET AaffReadNextPage (t_pAaff pAaff, unsigned char *pData, unsigned int *pDataLen)
+APIRET AaffReadNextPage (t_pAaff pAaff, unsigned char *pData, unsigned int *pDataLen, QString *pImageFilename, t_pHashMD5Digest pMD5Digest, bool *pMD5Valid)
 {
    t_AffSegmentHeader Header;
    t_AffSegmentFooter Footer;
@@ -435,13 +476,16 @@ APIRET AaffReadNextPage (t_pAaff pAaff, unsigned char *pData, unsigned int *pDat
    bool               Found=false;
    uLongf             DataLen = *pDataLen;
    int                rc;
+   unsigned long long BytesToNextHeader;
+
+   *pImageFilename = QString();
+   *pMD5Valid      = false;
 
    // Search for the next segment whose name starts with "page"
    // ---------------------------------------------------------
    do
    {
-      if (fread  (&Header, offsetof(t_AffSegmentHeader, Name), 1, pAaff->pFile) != 1)
-         return ERROR_AAFF_CANNOT_READ_HEADER;
+      CHK (AaffReadFile (pAaff, &Header, offsetof(t_AffSegmentHeader, Name)))
       if (strcmp (&Header.Magic[0], AFF_SEGMENT_HEADER_MAGIC) != 0)
          return ERROR_AAFF_INVALID_HEADER;
       Header.NameLen  = ntohl (Header.NameLen );
@@ -451,8 +495,7 @@ APIRET AaffReadNextPage (t_pAaff pAaff, unsigned char *pData, unsigned int *pDat
       if (Header.NameLen >= sizeof(SegmentName))
          return ERROR_AAFF_INVALID_SEGMENT_NAME;
 
-      if (fread  (&SegmentName[0], Header.NameLen, 1, pAaff->pFile) != 1)
-         return ERROR_AAFF_CANNOT_READ_HEADER;
+      CHK (AaffReadFile (pAaff, &SegmentName[0], Header.NameLen))
       SegmentName[Header.NameLen] = '\0';
 
       Found            = (strncmp (&SegmentName[0], AFF_SEGNAME_PAGE, strlen(AFF_SEGNAME_PAGE)) == 0); // The segment name must start with "page"
@@ -461,9 +504,18 @@ APIRET AaffReadNextPage (t_pAaff pAaff, unsigned char *pData, unsigned int *pDat
 
       if (!Found)
       {
-         rc = fseeko64 (pAaff->pFile, Header.DataLen+sizeof(t_AffSegmentFooter), SEEK_CUR);
-         if (rc)
-            return ERROR_AAFF_CANNOT_SEEK;
+         BytesToNextHeader = Header.DataLen + sizeof(t_AffSegmentFooter);
+         if (CONFIG (CalcImageFileMD5))          // Read the data between current pos and next header in order have image file MD5 calculated correctly
+         {
+            CHK (AaffReallocUncompressBuffer (pAaff, BytesToNextHeader))
+            CHK (AaffReadFile                (pAaff, pAaff->pUncompressBuffer, BytesToNextHeader))
+         }
+         else                                   // Simply seek to next header, which is faster
+         {
+            rc = fseeko64 (pAaff->pFile, BytesToNextHeader, SEEK_CUR);
+            if (rc)
+               return ERROR_AAFF_CANNOT_SEEK;
+         }
       }
    } while (!Found);
 
@@ -483,29 +535,19 @@ APIRET AaffReadNextPage (t_pAaff pAaff, unsigned char *pData, unsigned int *pDat
    switch (Header.Argument)
    {
       case AFF_PAGEFLAGS_UNCOMPRESSED:
-         if (fread (pData, Header.DataLen, 1, pAaff->pFile) != 1)
-            return ERROR_AAFF_CANNOT_READ_DATA;
+         CHK (AaffReadFile (pAaff, pData, Header.DataLen))
          *pDataLen = Header.DataLen;
          break;
       case AFF_PAGEFLAGS_COMPRESSED_ZERO:
          unsigned int Len;
-         if (fread (&Len, sizeof(Len), 1, pAaff->pFile) != 1)
-            return ERROR_AAFF_CANNOT_READ_DATA;
+         CHK (AaffReadFile (pAaff, &Len, sizeof(Len)))
          Len = ntohl (Len);
          memset (pData, 0, Len);
          *pDataLen = Len;
          break;
       case AFF_PAGEFLAGS_COMPRESSED_ZLIB:
-         if (pAaff->UncompressBufferLen < Header.DataLen)
-         {
-            pAaff->pUncompressBuffer = (unsigned char *) realloc (pAaff->pUncompressBuffer, Header.DataLen);
-            if (pAaff->pUncompressBuffer == NULL)
-               CHK (ERROR_AAFF_MEMALLOC_FAILED)
-            pAaff->UncompressBufferLen = Header.DataLen;
-         }
-         if (fread (pAaff->pUncompressBuffer, Header.DataLen, 1, pAaff->pFile) != 1)
-            return ERROR_AAFF_CANNOT_READ_DATA;
-
+         CHK (AaffReallocUncompressBuffer (pAaff, Header.DataLen))
+         CHK (AaffReadFile                (pAaff, pAaff->pUncompressBuffer, Header.DataLen))
          rc = uncompress (pData, &DataLen, pAaff->pUncompressBuffer, Header.DataLen);
          *pDataLen = DataLen;
          if (rc != Z_OK)
@@ -521,8 +563,7 @@ APIRET AaffReadNextPage (t_pAaff pAaff, unsigned char *pData, unsigned int *pDat
 
    // Check footer
    // ------------
-   if (fread (&Footer, sizeof(Footer), 1, pAaff->pFile) != 1)
-      return ERROR_AAFF_CANNOT_READ_FOOTER;
+   CHK (AaffReadFile (pAaff, &Footer, sizeof(Footer)))
    if (strcmp (&Footer.Magic[0], AFF_SEGMENT_FOOTER_MAGIC) != 0)
       return ERROR_AAFF_INVALID_FOOTER;
 
@@ -530,43 +571,62 @@ APIRET AaffReadNextPage (t_pAaff pAaff, unsigned char *pData, unsigned int *pDat
    if (ntohl(Footer.SegmentLen) != SegmentLen)
       return ERROR_AAFF_INVALID_SEGMENTLEN;
 
+   // Check if all data has been read
+   // -------------------------------
+   pAaff->BytesVerified += *pDataLen;
+   if (pAaff->BytesVerified == pAaff->DeviceSize)
+   {
+      if (CONFIG(CalcImageFileMD5))
+      {
+         unsigned long long ImageFileSize;
+         unsigned long long BytesRemaining;
+
+         CHK (AaffGetImageFileSize(pAaff, &ImageFileSize))
+         BytesRemaining = ImageFileSize - AaffGetCurrentSeekPos(pAaff);
+         CHK (AaffReadFile (pAaff, pAaff->pUncompressBuffer, BytesRemaining))
+         CHK (HashMD5Digest (&pAaff->HashContextMD5, pMD5Digest))
+         *pMD5Valid = true;
+      }
+      *pImageFilename = *(pAaff->pFilename);
+   }
+
    return NO_ERROR;
 }
 
 
 // -----------------------
-//  Module initialisation
+//      Misc. functions
 // -----------------------
 
+unsigned int AaffPreprocessExtraSpace (unsigned int FifoBlockSize)
+{
+   return UtilGetMaxZcompressedBufferSize (FifoBlockSize) - FifoBlockSize;
+}
+
 APIRET AaffInit (void)
 {
    int i;
 
-   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_MEMALLOC_FAILED        ))
-   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_CANNOT_CREATE_FILE     ))
-   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_CANNOT_OPEN_FILE       ))
-   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_CANNOT_WRITE_FILE      ))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_CANNOT_CLOSE_FILE      ))
+   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_CANNOT_CREATE_FILE     ))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_CANNOT_FLUSH_FILE      ))
-   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_CANNOT_READ_SIGNATURE  ))
-   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_CANNOT_READ_HEADER     ))
-   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_CANNOT_READ_FOOTER     ))
+   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_CANNOT_OPEN_FILE       ))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_CANNOT_READ_DATA       ))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_CANNOT_SEEK            ))
-   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_SECTORSIZE_TOO_BIG     ))
-   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_COMPRESSION_FAILED     ))
-   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_INVALID_SIGNATURE      ))
-   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_INVALID_HEADER         ))
+   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_CANNOT_WRITE_FILE      ))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_INVALID_FOOTER         ))
-   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_INVALID_SEGMENTLEN     ))
-   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_INVALID_PAGE_ORDER     ))
+   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_INVALID_HEADER         ))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_INVALID_PAGE_ARGUMENT  ))
-   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_INVALID_SEGMENT_NAME   ))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_INVALID_PAGE_NUMBER    ))
+   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_INVALID_PAGE_ORDER     ))
+   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_INVALID_SEGMENTLEN     ))
+   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_INVALID_SEGMENT_NAME   ))
+   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_INVALID_SIGNATURE      ))
+   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_MEMALLOC_FAILED        ))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_PAGE_LARGER_THAN_BUFFER))
+   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_SECTORSIZE_TOO_BIG     ))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_AAFF_UNCOMPRESS_FAILED      ))
 
-
    srandom (time (0));
 
    for (i=0; i<AAFF_BADSECTORMARKER_MAXLEN; i++)
