@@ -9,7 +9,7 @@
 //  Module:         Multithreaded AEWF (AEWF = Advanced EWF)
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -52,11 +52,13 @@
 // Qt inconsistently defines its little/big endian functions, see QTBUG-21208 on bugreports.qt.nokia.com
 // The following lines implement conversin for uchar and avoid the problems. 
 
-template <typename T> T qbswap(T source);
-template <> inline uchar qbswap<uchar>(uchar source)
-{
-   return source;
-}
+#if (QT_VERSION < 0x040800)
+   template <typename T> T qbswap(T source);
+   template <> inline uchar qbswap<uchar>(uchar source)
+   {
+      return source;
+   }
+#endif
 
 // ------------------
 //  AEWF definitions
