@@ -9,7 +9,7 @@
 //  Module:         Application configuration data
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -29,7 +29,7 @@
 #include "common.h"
 
 #include <limits.h>
-#include "proc/sysinfo.h"  // Required in order to get smp_num_cpus for finding out the number of CPUs in the system
+//#include "proc/sysinfo.h"  // Required in order to get smp_num_cpus for finding out the number of CPUs in the system
 
 #include <qcolor.h>
 #include <qfont.h>
@@ -41,6 +41,7 @@
 #include "file.h"
 
 #include "qtutil.h"
+#include "util.h"
 #include "config.h"
 
 // ------------------------------------
@@ -58,7 +59,7 @@ static const char * CFG_SECTION_GUYMAGER = "GUYMAGER";
 
 static const unsigned char CONFIG_DUMMY_FILL = 0xAA;
 
-static const int CFG_MAX_COMPRESSIONTHREADS = 16;
+static const uint CFG_MAX_COMPRESSIONTHREADS = 16;
 
 // ------------------------------------
 //          Type definitions
@@ -345,6 +346,7 @@ static t_ToolCfgParamDesc CfgParamDescArr[] =
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"ScanInterval"                  , CFGTYPE_INTEGER, ELT(ScanInterval                  ),                0,       1, 8640000, NULL                            }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"CommandGetSerialNumber"        , CFGTYPE_STRING , ELT_SIZ(CommandGetSerialNumber    ),                         0,       0, NULL                            }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"CommandGetAddStateInfo"        , CFGTYPE_STRING , ELT_SIZ(CommandGetAddStateInfo    ),                         0,       0, NULL                            }, CFG_FILLUP_FORLINT},
+   {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"QueryDeviceMediaInfo"          , CFGTYPE_SET    , ELT(QueryDeviceMediaInfo          ),                0,       0,       0, INIARR(SetArrBoolean)           }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"DirectIO"                      , CFGTYPE_SET    , ELT(DirectIO                      ),                0,       0,       0, INIARR(SetArrBoolean)           }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"FifoBlockSizeDD"               , CFGTYPE_INTEGER, ELT(FifoBlockSizeDD               ),                0,       0,99999999, NULL                            }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"FifoBlockSizeEWF"              , CFGTYPE_INTEGER, ELT(FifoBlockSizeEWF              ),                0,       0,99999999, NULL                            }, CFG_FILLUP_FORLINT},
@@ -594,16 +596,19 @@ APIRET CfgIniLang (t_pToolCfgParamDesc pCfgParamDesc, t_pchar *ppErrorText)
 
 APIRET CfgIniCPUs (t_pToolCfgParamDesc pCfgParamDesc, t_pchar *ppErrorText)
 {
-   t_pcchar pParamName;
+   t_pcchar     pParamName;
+   unsigned int  CPUs;
 
    *ppErrorText = NULL;
    pParamName = pCfgParamDesc->DataDesc.pName;
 
    if (CONFIG (CompressionThreads) == CONFIG_COMPRESSIONTHREADS_AUTO)
    {
-      LOG_INFO ("Parameter %s set to AUTO; %ld CPUs detected", pParamName, smp_num_cpus)
-      CONFIG (CompressionThreads) = smp_num_cpus;
-      if (smp_num_cpus > CFG_MAX_COMPRESSIONTHREADS)
+//      CPUs = smp_num_cpus;
+      CPUs = UtilGetNumberOfCPUs();
+      LOG_INFO ("Parameter %s set to AUTO; %u CPUs detected", pParamName, CPUs)
+      CONFIG (CompressionThreads) = CPUs;
+      if (CPUs > CFG_MAX_COMPRESSIONTHREADS)
       {
          CONFIG (CompressionThreads) = CFG_MAX_COMPRESSIONTHREADS;
          LOG_INFO ("Maximum value for %s is %d", pParamName, CFG_MAX_COMPRESSIONTHREADS)
@@ -616,17 +621,22 @@ APIRET CfgIniCPUs (t_pToolCfgParamDesc pCfgParamDesc, t_pchar *ppErrorText)
 
 APIRET CfgIniMem (t_pToolCfgParamDesc pCfgParamDesc, t_pchar *ppErrorText)
 {
-   t_pcchar pParamName;
+   t_pcchar           pParamName;
+   unsigned long long  Bytes;
 
    *ppErrorText = NULL;
    pParamName = pCfgParamDesc->DataDesc.pName;
 
-   meminfo();
+//   meminfo();
+//   Bytes = kb_main_total * 1024;
+
+   Bytes = UtilGetInstalledRAM ();
+
    if (CONFIG (FifoMaxMem) == 0)
    {
-      CONFIG (FifoMaxMem) = GETMAX (1, (int)(kb_main_total / (8*1024)));   // Use one eighth of the available mem, convert KB to MB
-      CONFIG (FifoMaxMem) = GETMIN (CONFIG (FifoMaxMem), 64);              // Stay below 64MB
-      LOG_INFO ("Parameter %s set to 0 (auto); %lu MB of RAM detected.", pParamName, kb_main_total/1024)
+      CONFIG (FifoMaxMem) = GETMAX (1, (int)(Bytes / (8*1024*1024)));   // Use one eighth of the available mem, convert to MB
+      CONFIG (FifoMaxMem) = GETMIN (CONFIG (FifoMaxMem), 64);       // Stay below 64MB
+      LOG_INFO ("Parameter %s set to 0 (auto); %0.1f MB of RAM detected.", pParamName, Bytes/(1024.0*1024.0));
       LOG_INFO ("Setting %s to %d (i.e. using %d MB per acquisition as FIFO memory)", pParamName, CONFIG (FifoMaxMem), CONFIG (FifoMaxMem))
    }
 
