@@ -11,7 +11,7 @@
 //                  window.
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -105,6 +105,7 @@ void t_Device::Initialise (void)
    memset (&this->SHA256Digest         , 0, sizeof(this->SHA256Digest         ));
    memset (&this->SHA256DigestVerifySrc, 0, sizeof(this->SHA256DigestVerifySrc));
    memset (&this->SHA256DigestVerifyDst, 0, sizeof(this->SHA256DigestVerifyDst));
+// this->ImageFileHashes
 
    this->pFifoCompressIn      = NULL;
    this->pFifoCompressOut     = NULL;
@@ -127,11 +128,16 @@ void t_Device::Initialise (void)
 //   this->Acquisition.Description   = QString();
 //   this->Acquisition.Notes         = QString();
 
+   this->AddStateInfo.CanBeAcquired = true   ;
+   this->AddStateInfo.Color         = COLOR_DEFAULT;
+// this->AddStateInfo.Info          = QString();
+   
    this->Info.SetDevice(this);
 
-   this->PrevTimestamp     = QTime();
-   this->PrevSpeed         = 0.0;
-   this->Checked           = false;
+   this->PrevTimestamp = QTime();
+   this->PrevSpeed     = 0.0;
+   this->Checked       = false;
+   this->AddedNow      = false;
 }
 
 void t_Device::InitialiseDeviceSpecific (const QString &SerialNumber, const QString &LinuxDevice, const QString &Model,
@@ -146,10 +152,13 @@ void t_Device::InitialiseDeviceSpecific (const QString &SerialNumber, const QStr
    this->SectorSizePhys = SectorSizePhys;
    this->Size           = Size;
 
+//   #define FAKE_DEVICE_SIZE
    #ifdef FAKE_DEVICE_SIZE
-      // this->Size = GETMIN (10737418240ULL, Size);  // 10 GiB For faster testing (we make guymager believe that the device is smaller)
-      // this->Size = GETMIN (   20485760ULL, Size);  // 20 MiB
-      // this->Size = GETMIN ( 1000000000ULL, Size);  //  1 GB
+   #warning "Debug code for device size faking still enabled"
+       this->Size = GETMIN ( 10*1024, Size);               // 10K
+      // this->Size = GETMIN ( 20971520ULL, Size);           // 20 MiB
+      // this->Size = GETMIN ( 1024*1024*1024ULL, Size);     //  1 GB
+      // this->Size = GETMIN (10737418240ULL, Size);         // 10 GiB
       // this->Size = 3ULL*1024ULL*1024ULL*1024ULL*1024ULL;  //  3 TB
       printf ("\nSize of all devices faked to %llu bytes", this->Size);
    #endif
@@ -187,8 +196,9 @@ bool t_Device::HasHashThread (void) const
 
 bool t_Device::HasCompressionThreads (void) const
 {
-   return (CONFIG (CompressionThreads) > 0) && ((Acquisition.Format == t_File::EWF) ||
-                                                (Acquisition.Format == t_File::AFF));
+   return (CONFIG (CompressionThreads) > 0) && ((Acquisition.Format == t_File::EWF ) ||
+                                                (Acquisition.Format == t_File::AAFF) ||
+                                                (Acquisition.Format == t_File::AEWF));
 }
 
 
@@ -199,6 +209,11 @@ QVariant t_Device::GetSerialNumber (t_pDevice pDevice)
    return pDevice->SerialNumber;
 }
 
+QVariant t_Device::GetAddStateInfo (t_pDevice pDevice)
+{
+   return pDevice->AddStateInfo.Info;
+}
+
 QVariant t_Device::GetLinuxDevice (t_pDevice pDevice)
 {
    return pDevice->LinuxDevice;
@@ -216,10 +231,22 @@ QVariant t_Device::GetState (t_pDevice pDevice)
    switch (pDevice->State)
    {
       case Idle         : if (pDevice->Local)
-                                    StrState = t_MainWindow::tr("Local device");
-                          else if (MainGetDeviceList()->UsedAsCloneDestination(pDevice))
-                                    StrState = t_MainWindow::tr("Used in clone operation");
-                               else StrState = t_MainWindow::tr("Idle");
+                          {
+                             StrState = t_MainWindow::tr("Local device");
+                          }
+                          else 
+                          {
+                             if (MainGetDeviceList()->UsedAsCloneDestination(pDevice))
+                             {
+                                StrState = t_MainWindow::tr("Used in clone operation");
+                             }
+                             else
+                             {
+                                StrState = t_MainWindow::tr("Idle");
+                                if (CONFIG (CommandGetAddStateInfo)[0])
+                                   StrState += " - " + pDevice->AddStateInfo.Info;
+                             }
+                          }
                           break;
       case Acquire      : if (pDevice->AbortRequest)
                                StrState = t_MainWindow::tr("Aborting..."        );
@@ -238,7 +265,7 @@ QVariant t_Device::GetState (t_pDevice pDevice)
                                 case  6: StrState += " just shut up"; break;
                                 case  7: StrState += " SHUT UP!"; break;
                                 case  8: StrState += " you want a segmentation fault?"; break;
-                                case  9: StrState += " you can have it if you want"; break;
+                                case  9: StrState += " you can have one if you want"; break;
                                 case 10: StrState += " I warn you"; break;
                                 case 11: StrState += " last warning"; break;
                                 case 12: StrState += " ultimate warning"; break;
@@ -285,6 +312,8 @@ QVariant t_Device::GetState (t_pDevice pDevice)
                         case t_Device::ThreadWriteWriteError : StrState = t_MainWindow::tr("Aborted - Image file write error" );  break;
                         case t_Device::ThreadWriteVerifyError: StrState = t_MainWindow::tr("Aborted - Image file verify error" ); break;
                         case t_Device::ThreadReadFileError   : StrState = t_MainWindow::tr("Aborted - Device read error" );       break;
+                        case t_Device::InfoWriteError        : StrState = t_MainWindow::tr("Aborted - Info file write error");    break;
+                        case t_Device::AcquisitionStartFailed: StrState = t_MainWindow::tr("Aborted - Acquisition start failure");break;
                         default                              : CHK_EXIT (ERROR_DEVICE_BAD_ABORTREASON)
                      }
                      break;
@@ -644,7 +673,15 @@ APIRET t_DeviceList::MatchDevice (t_pcDevice pDevCmp, t_pDevice &pDevMatch)
       }         
       else  // Not a special device, we can use normal matching
       {
-         if (pDevMatch->SerialNumber.isEmpty())
+// Lines below commented out, as somebody might change assignement of the devices starting with the names shown below after Guymager has done a device scan
+//         if ((pDevMatch->LinuxDevice.startsWidth("/dev/loop") ||  // These are devices generated internally by the loop device driver,
+//             (pDevMatch->LinuxDevice.startsWidth("/dev/dm-" ) ||  // LVM or DM. we can be sure that they have unique names, so let's
+//             (pDevMatch->LinuxDevice.startsWidth("/dev/md"  ))    // avoid all the problems with empty serial numbers and so on.
+//         {
+//            Found = (pDevMatch->LinuxDevice == pDevCmp->LinuxDevice);
+//         }
+//         else
+              if (pDevMatch->SerialNumber.isEmpty())
          {
             Found = (pDevCmp->SerialNumber.isEmpty() && (pDevMatch->State != t_Device::AcquirePaused) &&
                                                         (pDevMatch->State != t_Device::VerifyPaused ) &&
