@@ -7,7 +7,7 @@
 //                  Section Nouvelles Technologies
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -39,6 +39,7 @@
 #include "info.h"
 #include "file.h"
 #include "media.h"
+#include "config.h"
 
 class t_ThreadRead;
 class t_ThreadHash;
@@ -69,7 +70,9 @@ class t_Device
          UserRequest,
          ThreadWriteWriteError,
          ThreadWriteVerifyError,
-         ThreadReadFileError
+         ThreadReadFileError,
+         InfoWriteError,
+         AcquisitionStartFailed
       } t_AbortReason;
 
       static const int AutoFracLen       = -1;   // Use up to 2 digits after decimal point (min. 3 significant digits)
@@ -97,6 +100,39 @@ class t_Device
             quint64        SplitFileSize;    // 0 means do not split
       };
 
+      class t_ImageFileHash // Holds the hash of a complete image file (not just the image contents)
+      {
+         public:
+            t_ImageFileHash (const QString &Filename, t_pHashMD5Digest pMD5Digest=NULL, bool MD5ok=false)
+            {
+               this->Filename = Filename;
+               MD5Valid = (pMD5Digest != NULL) && MD5ok;
+               if (MD5Valid)
+                  MD5Digest = *pMD5Digest;
+            }
+
+         public:
+            QString         Filename;
+            bool            MD5Valid;
+            t_HashMD5Digest MD5Digest;
+      };
+
+      class t_AddStateInfo       // Allows the user to add state information to a device
+      {
+          public:
+             t_AddStateInfo () : CanBeAcquired(true),
+                                 Color(COLOR_DEFAULT)
+             {};
+
+          public:
+             QString Info;
+             bool    CanBeAcquired;
+             int     Color;         // Refers to the color AdditionalState1, AdditionalState2, ... in the configuration file.
+      };
+
+
+      typedef QList<t_ImageFileHash *> t_ImageFileHashList;
+
    public:
       t_Device (const QString &SerialNumber=QString(), const PedDevice *pPedDev=NULL);
       t_Device (const QString &SerialNumber, const QString &LinuxDevice, const QString &Model,
@@ -107,6 +143,7 @@ class t_Device
       static QVariant GetLinuxDevice         (t_pDevice pDevice);
       static QVariant GetModel               (t_pDevice pDevice);
       static QVariant GetState               (t_pDevice pDevice);
+      static QVariant GetAddStateInfo        (t_pDevice pDevice);
       static QVariant GetSectorSize          (t_pDevice pDevice);
       static QVariant GetSectorSizePhys      (t_pDevice pDevice);
       static QVariant GetBadSectorCount      (t_pDevice pDevice);
@@ -268,7 +305,8 @@ class t_Device
 
             t_State                   State;
    private: QString                   Message;          // Used by the threads to communicate messages displayed in spreadsheet field "state"
-   public:  bool                      AbortRequest;
+   public:  t_AddStateInfo            AddStateInfo;
+            bool                      AbortRequest;
             int                       AbortCount;       // Little easter egg  ;-)
             t_AbortReason             AbortReason;
             bool                      DeleteAfterAbort;
@@ -311,11 +349,13 @@ class t_Device
             t_HashSHA256Digest        SHA256Digest;
             t_HashSHA256Digest        SHA256DigestVerifySrc;
             t_HashSHA256Digest        SHA256DigestVerifyDst;
+            t_ImageFileHashList       ImageFileHashList;
 
    public:  quint64                   PrevPos;          // Some variables for
             QTime                     PrevTimestamp;    // the current speed
             double                    PrevSpeed;        // calculation.
-            bool                      Checked;          // Helper variable for matching algorithm in SlotScanFinished, not used elsewhere.
+            bool                      Checked;          // Helper variables for matching algorithm ...
+            bool                      AddedNow;         // ... in SlotScanFinished, not used elsewhere.
 
    private: QReadWriteLock            SemCurrentWritePos;
    private: QReadWriteLock            SemCurrentVerifyPosSrc;
