@@ -9,7 +9,7 @@
 //  Module:         Acquisition dialog
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -70,6 +70,9 @@ class t_DlgAcquireLocal
       QPushButton     *pButtonCancel;
 
       t_pDevice        pDevice;
+
+      bool              OldSplitFileSwitch;              // Required to remember SplitFileSwitch when chaging forth and back
+                                                         // between formats that support split files and not.
 };
 
 static t_File::Format DlgAcquireLastUsedFormat = t_File::NotSet;  // For remembering the format that was used for the last acquisition
@@ -617,13 +620,13 @@ t_DlgAcquire::t_DlgAcquire (t_pDevice pDevice, bool Clone, t_pDeviceList pDevice
       pLayoutFormatSplit->addLayout (pLayoutSplit);
       pLayoutGroupBox   ->addLayout (pLayoutEwf);
 
-      CHK_EXIT (t_File::GetFormatDescription (t_File::DD , false, 0, Str))            ButtonTextDD  = "&" + Str;
-      CHK_EXIT (t_File::GetFormatDescription (t_File::EWF, false, 0, Str))            ButtonTextEWF = "&" + Str;
-      CHK_EXIT (t_File::GetFormatDescription (t_File::AFF, false, 0, Str))            ButtonTextAFF = "&" + Str;
-      CHK_EXIT (t_File::GetFormatExtension   (t_File::DD , false, 0, NULL, &Str))
-      CHK_EXIT (t_File::GetFormatExtension   (t_File::DD , false, 3, NULL, &StrAlt))  ButtonTextDD += " " + tr("(file extension %1 or %2)") .arg (Str) .arg (StrAlt);
-      CHK_EXIT (t_File::GetFormatExtension   (t_File::EWF, false, 0, NULL, &Str))     ButtonTextEWF+= " " + tr("(file extension %1)") .arg (Str);
-      CHK_EXIT (t_File::GetFormatExtension   (t_File::AFF, false, 0, NULL, &Str))     ButtonTextAFF+= " " + tr("(file extension %1)") .arg (Str);
+      CHK_EXIT (t_File::GetFormatDescription (t_File::DD  , false, 0, Str))            ButtonTextDD  = "&" + Str;
+      CHK_EXIT (t_File::GetFormatDescription (t_File::EWF , false, 0, Str))            ButtonTextEWF = "&" + Str;
+      CHK_EXIT (t_File::GetFormatDescription (t_File::AAFF, false, 0, Str))            ButtonTextAFF = "&" + Str;
+      CHK_EXIT (t_File::GetFormatExtension   (t_File::DD  , false, 0, NULL, &Str))
+      CHK_EXIT (t_File::GetFormatExtension   (t_File::DD  , false, 3, NULL, &StrAlt))  ButtonTextDD += " " + tr("(file extension %1 or %2)") .arg (Str) .arg (StrAlt);
+      CHK_EXIT (t_File::GetFormatExtension   (t_File::EWF , false, 0, NULL, &Str))     ButtonTextEWF+= " " + tr("(file extension %1)") .arg (Str);
+      CHK_EXIT (t_File::GetFormatExtension   (t_File::AAFF, false, 0, NULL, &Str))     ButtonTextAFF+= " " + tr("(file extension %1)") .arg (Str);
 
       pOwn->pRadioButtonFormatDD  = new QRadioButton (ButtonTextDD , this);
       pOwn->pRadioButtonFormatEWF = new QRadioButton (ButtonTextEWF, this);
@@ -648,6 +651,7 @@ t_DlgAcquire::t_DlgAcquire (t_pDevice pDevice, bool Clone, t_pDeviceList pDevice
       CHK_EXIT (AddField (pDevice, pFieldSplitFileSwitch, pLayoutSplit, &Row, &Col))
       CHK_EXIT (AddField (pDevice, pFieldSplitFileSize  , pLayoutSplit, &Row, &Col))
       CHK_EXIT (AddField (pDevice, pFieldSplitFileUnit  , pLayoutSplit, &Row, &Col))
+      pOwn->OldSplitFileSwitch = pFieldSplitFileSwitch->pCheckBox->isChecked();
 
       Row = Col = 0;
       for (i=0; i<pDlgAcquireFields->count(); i++)
@@ -729,10 +733,10 @@ t_DlgAcquire::t_DlgAcquire (t_pDevice pDevice, bool Clone, t_pDeviceList pDevice
 
       switch (DlgAcquireLastUsedFormat)
       {
-         case t_File::DD : pOwn->pRadioButtonFormatDD ->setChecked (true); break;
-         case t_File::EWF: pOwn->pRadioButtonFormatEWF->setChecked (true); break;
-         case t_File::AFF: pOwn->pRadioButtonFormatAFF->setChecked (true); break;
-         default         : CHK_EXIT (ERROR_DLGACQUIRE_INVALID_FORMAT)
+         case t_File::DD  : pOwn->pRadioButtonFormatDD ->setChecked (true); break;
+         case t_File::EWF : pOwn->pRadioButtonFormatEWF->setChecked (true); break;
+         case t_File::AAFF: pOwn->pRadioButtonFormatAFF->setChecked (true); break;
+         default          : CHK_EXIT (ERROR_DLGACQUIRE_INVALID_FORMAT)
       }
    }
 
@@ -742,7 +746,7 @@ t_DlgAcquire::t_DlgAcquire (t_pDevice pDevice, bool Clone, t_pDeviceList pDevice
    // Adjust size
    // -----------
    // Qt limits dialogs to 2/3 of the screen height and width. That's why we adjust it with
-   //  the following code if the dialog is smaller then its size hint.
+   // the following code if the dialog is smaller then its size hint.
    adjustSize();  // Let Qt calculate the size (limited to 2/3 of the screen)
    QRect DialogRect = geometry ();
    
@@ -842,7 +846,7 @@ void t_DlgAcquire::SlotDeviceTableSelectionChanged (void)
    {
       if (Selected != pOwn->pDeviceTable->columnCount())
       {
-         LOG_INFO ("%d devices in device tabel for clone destination", Selected)
+         LOG_INFO ("%d devices in device table for clone destination", Selected)
          CHK_EXIT (ERROR_DLGACQUIRE_INVALID_SELECTION)
       }
       Row   = SelectedItemsList[0]->row();
@@ -880,27 +884,26 @@ void t_DlgAcquire::UpdateFieldState (void)
 
    // Split file fields
    // -----------------
-   static bool OldSplitFileSwitch = false;  // should be replaced
-
    t_pCfgDlgAcquireField pFieldSplitFileSwitch = DlgAcquireGetField (CFG_DLGACQUIRE_SPLITFILESWITCH);
    t_pCfgDlgAcquireField pFieldSplitFileSize   = DlgAcquireGetField (CFG_DLGACQUIRE_SPLITFILESIZE  );
    t_pCfgDlgAcquireField pFieldSplitFileUnit   = DlgAcquireGetField (CFG_DLGACQUIRE_SPLITFILEUNIT  );
+
    if (pOwn->pRadioButtonFormatDD->isChecked())
    {
-      pFieldSplitFileSwitch->pCheckBox->setChecked (OldSplitFileSwitch);
+      pFieldSplitFileSwitch->pCheckBox->setChecked (pOwn->OldSplitFileSwitch);
       pFieldSplitFileSwitch->pCheckBox->setEnabled (true);
    }
    else if (pOwn->pRadioButtonFormatEWF->isChecked())
    {
       if (pFieldSplitFileSwitch->pCheckBox->isEnabled())
-         OldSplitFileSwitch = pFieldSplitFileSwitch->pCheckBox->isChecked();
+         pOwn->OldSplitFileSwitch = pFieldSplitFileSwitch->pCheckBox->isChecked();
       pFieldSplitFileSwitch->pCheckBox->setChecked (true);
       pFieldSplitFileSwitch->pCheckBox->setEnabled (false);
    }
    else if (pOwn->pRadioButtonFormatAFF->isChecked())
    {
       if (pFieldSplitFileSwitch->pCheckBox->isEnabled())
-         OldSplitFileSwitch = pFieldSplitFileSwitch->pCheckBox->isChecked();
+         pOwn->OldSplitFileSwitch = pFieldSplitFileSwitch->pCheckBox->isChecked();
       pFieldSplitFileSwitch->pCheckBox->setChecked (false);
       pFieldSplitFileSwitch->pCheckBox->setEnabled (false);
    }
@@ -1120,12 +1123,13 @@ void t_DlgAcquire::SlotAccept (void)
             return;
          }
          SplitSize = NumValue * UnitIndexToMultiplier (DlgAcquireGetField(CFG_DLGACQUIRE_SPLITFILEUNIT)->pComboBox->currentIndex());
-         #if (LIBEWF_VERSION != 20100226)
+         #if ((LIBEWF_VERSION != 20100226) && (LIBEWF_VERSION != 20080501) && (LIBEWF_VERSION != 20111015))
             #error "Please check EWF documentation for newer Encase formats and adjust following code"
          #endif
          if (pOwn->pRadioButtonFormatEWF->isChecked())
          {
-            if (CONFIG(EwfFormat) == LIBEWF_FORMAT_ENCASE6)
+            if ((CONFIG(EwfFormat) == LIBEWF_FORMAT_ENCASE6) ||
+                (CONFIG(EwfFormat) == t_File::AEWF))
             {
                if ((SplitSize > EWF_MAX_SEGMENT_SIZE_EXT) || (SplitSize < EWF_MIN_SEGMENT_SIZE))
                {
@@ -1138,10 +1142,10 @@ void t_DlgAcquire::SlotAccept (void)
             }
             else
             {
-               if ((NumValue > EWF_MAX_SEGMENT_SIZE) || (NumValue < EWF_MIN_SEGMENT_SIZE))
+               if ((SplitSize > EWF_MAX_SEGMENT_SIZE) || (SplitSize < EWF_MIN_SEGMENT_SIZE))
                {
                   QMessageBox::information (this, tr ("Incorrect value", "Dialog title"),
-                                                  tr ("The split size for the configured EWF format must be in the range %1 - %2 MiB."
+                                                  tr ("The split size for the configured EWF format must be in the range %1 - %2 MiB. "
                                                       "For bigger files, switch to ""Encase6"" format or later (see Guymager configuration file, parameter EwfFormat).")
                                                   .arg(EWF_MIN_SEGMENT_SIZE/(1024*1024))
                                                   .arg(EWF_MAX_SEGMENT_SIZE/(1024*1024)), QMessageBox::Ok);
@@ -1272,7 +1276,7 @@ APIRET t_DlgAcquire::GetParameters (t_Device::t_Acquisition &Acquisition, bool R
    {
       if      (pOwn->pRadioButtonFormatDD ->isChecked()) Acquisition.Format = t_File::DD;
       else if (pOwn->pRadioButtonFormatEWF->isChecked()) Acquisition.Format = t_File::EWF;
-      else                                               Acquisition.Format = t_File::AFF;
+      else                                               Acquisition.Format = t_File::AAFF;
       if (RememberLastUsedValues)
          DlgAcquireLastUsedFormat = Acquisition.Format;
    }
@@ -1285,11 +1289,11 @@ APIRET t_DlgAcquire::GetParameters (t_Device::t_Acquisition &Acquisition, bool R
          pDlgAcquireField->LastEnteredValue = Acq;       \
    }
 
-   #define COPY_CHECKBOX(Acq, FieldName)                 \
+   #define COPY_CHECKBOX(Acq, FieldName, Remember)       \
    {                                                     \
       pDlgAcquireField = DlgAcquireGetField (FieldName); \
       Acq = pDlgAcquireField->pCheckBox->isChecked();    \
-      if (RememberLastUsedValues)                        \
+      if (Remember)                                      \
          pDlgAcquireField->LastEnteredValue = Acq  ? "1" : "0"; \
    }
 
@@ -1303,13 +1307,12 @@ APIRET t_DlgAcquire::GetParameters (t_Device::t_Acquisition &Acquisition, bool R
 
    if (!pOwn->Clone)
    {
-      Acquisition.SplitFileSize = 0;
       COPY_LINEENTRY (Acquisition.CaseNumber    , CFG_DLGACQUIRE_EWF_CASENUMBER     , RememberLastUsedValues)
       COPY_LINEENTRY (Acquisition.EvidenceNumber, CFG_DLGACQUIRE_EWF_EVIDENCENUMBER , RememberLastUsedValues)
       COPY_LINEENTRY (Acquisition.Examiner      , CFG_DLGACQUIRE_EWF_EXAMINER       , RememberLastUsedValues)
       COPY_LINEENTRY (Acquisition.Description   , CFG_DLGACQUIRE_EWF_DESCRIPTION    , RememberLastUsedValues)
       COPY_LINEENTRY (Acquisition.Notes         , CFG_DLGACQUIRE_EWF_NOTES          , RememberLastUsedValues)
-      COPY_CHECKBOX (SplitFileSwitch, CFG_DLGACQUIRE_SPLITFILESWITCH )
+      COPY_CHECKBOX  (SplitFileSwitch           , CFG_DLGACQUIRE_SPLITFILESWITCH    , RememberLastUsedValues)
       if (SplitFileSwitch)
       {
          int UnitIndex;
@@ -1317,16 +1320,20 @@ APIRET t_DlgAcquire::GetParameters (t_Device::t_Acquisition &Acquisition, bool R
          COPY_COMBOBOX  (UnitIndex    , CFG_DLGACQUIRE_SPLITFILEUNIT, RememberLastUsedValues)
          Acquisition.SplitFileSize = SplitFileSize.toULongLong() * UnitIndexToMultiplier(UnitIndex);
       }
+      else
+      {
+         Acquisition.SplitFileSize = 0;
+      }
    }
    COPY_LINEENTRY (Acquisition.ImagePath     , CFG_DLGACQUIRE_DEST_IMAGEDIRECTORY, pOwn->Clone ? false : RememberLastUsedValues)
    COPY_LINEENTRY (Acquisition.ImageFilename , CFG_DLGACQUIRE_DEST_IMAGEFILENAME , pOwn->Clone ? false : RememberLastUsedValues)
    COPY_LINEENTRY (Acquisition.InfoPath      , CFG_DLGACQUIRE_DEST_INFODIRECTORY , RememberLastUsedValues)
    COPY_LINEENTRY (Acquisition.InfoFilename  , CFG_DLGACQUIRE_DEST_INFOFILENAME  , RememberLastUsedValues)
 
-   COPY_CHECKBOX (Acquisition.CalcMD5   , CFG_DLGACQUIRE_HASH_CALC_MD5   )
-   COPY_CHECKBOX (Acquisition.CalcSHA256, CFG_DLGACQUIRE_HASH_CALC_SHA256)
-   COPY_CHECKBOX (Acquisition.VerifySrc , CFG_DLGACQUIRE_HASH_VERIFY_SRC )
-   COPY_CHECKBOX (Acquisition.VerifyDst , CFG_DLGACQUIRE_HASH_VERIFY_DST )
+   COPY_CHECKBOX (Acquisition.CalcMD5   , CFG_DLGACQUIRE_HASH_CALC_MD5   , RememberLastUsedValues)
+   COPY_CHECKBOX (Acquisition.CalcSHA256, CFG_DLGACQUIRE_HASH_CALC_SHA256, RememberLastUsedValues)
+   COPY_CHECKBOX (Acquisition.VerifySrc , CFG_DLGACQUIRE_HASH_VERIFY_SRC , RememberLastUsedValues)
+   COPY_CHECKBOX (Acquisition.VerifyDst , CFG_DLGACQUIRE_HASH_VERIFY_DST , RememberLastUsedValues)
 
    #undef COPY_LINEENTRY
    #undef COPY_CHECKBOX
