@@ -9,7 +9,7 @@
 //  Module:         Different message boxes we need all the time
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -88,6 +88,8 @@ void t_DlgMessage::AdjustGeometry (void)
    int X, Y;
    int Dx, Dy;
 
+   // Adjust size
+   // -----------
    // Trying to adjust the size of the dialog to size of the text inside. Not easy, as Qt doesn't provide the right functions for
    // doing so. Important: AdjustSize must be called after the dialog has been showed in order have QTextBox calculated all scroll
    // bar values.
@@ -103,8 +105,26 @@ void t_DlgMessage::AdjustGeometry (void)
    Dx = GETMIN (Dx, (DLGMESSAGE_MAX_SCREEN_PERCENTAGE * QApplication::desktop()->width ()) / 100);  // Limit to a certain amount
    Dy = GETMIN (Dy, (DLGMESSAGE_MAX_SCREEN_PERCENTAGE * QApplication::desktop()->height()) / 100);  // of the available screen space
 
-   X = (QApplication::desktop()->width  () - Dx) / 2;
-   Y = (QApplication::desktop()->height () - Dy) / 2;
+
+   // Adjust position
+   // ---------------
+   // Try to center relative to the parent if available, relative to srceen otherwise
+
+   QWidget *pParent = parentWidget();
+   if (pParent)
+   {
+      QPoint PosRel (pParent->x(), pParent->y());
+      QPoint PosAbs = pParent->mapToGlobal(PosRel);
+
+      X = PosAbs.x() + pParent->width ()/2 - Dx/2;
+      Y = PosAbs.y() + pParent->height()/2 - Dy/2;
+   }
+   else
+   {
+      X = (QApplication::desktop()->width  () - Dx) / 2;
+      Y = (QApplication::desktop()->height () - Dy) / 2;
+   }
+
    setGeometry (X, Y, Dx, Dy);
 }
 
@@ -113,11 +133,11 @@ t_DlgMessage::~t_DlgMessage ()
    delete pOwn;
 }
 
-APIRET t_DlgMessage::Show (const QString &Title, const QString &Message, bool Monospaced)
+APIRET t_DlgMessage::Show (const QString &Title, const QString &Message, bool Monospaced, QWidget *pParent, Qt::WFlags Flags)
 {
    t_DlgMessage *pDlg;
 
-   pDlg = new t_DlgMessage (Title, Message, Monospaced);
+   pDlg = new t_DlgMessage (Title, Message, Monospaced, pParent, Flags);
    pDlg->setModal      (true);
    pDlg->show          ();
    pDlg->AdjustGeometry();
