@@ -9,7 +9,7 @@
 //  Module:         Everything related to file names, extension names, paths...
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -69,21 +69,22 @@ APIRET t_File::GetFormatDescription (t_Format Format, bool Clone, int DdSplitDec
       case EWF:
          switch (CONFIG(EwfFormat))
          {
-            case LIBEWF_FORMAT_ENCASE1: SubFormat="Encase1"; break;
-            case LIBEWF_FORMAT_ENCASE2: SubFormat="Encase2"; break;
-            case LIBEWF_FORMAT_ENCASE3: SubFormat="Encase3"; break;
-            case LIBEWF_FORMAT_ENCASE4: SubFormat="Encase4"; break;
-            case LIBEWF_FORMAT_ENCASE5: SubFormat="Encase5"; break;
-            case LIBEWF_FORMAT_ENCASE6: SubFormat="Encase6"; break;
-            case LIBEWF_FORMAT_FTK    : SubFormat="FTK"    ; break;
-            case LIBEWF_FORMAT_SMART  : SubFormat="Smart"  ; break;
-            case LIBEWF_FORMAT_LVF    : SubFormat="LVF"    ; break;
-            case LIBEWF_FORMAT_LINEN5 : SubFormat="Linen5" ; break;
+            case LIBEWF_FORMAT_ENCASE1: SubFormat="Encase1" ; break;
+            case LIBEWF_FORMAT_ENCASE2: SubFormat="Encase2" ; break;
+            case LIBEWF_FORMAT_ENCASE3: SubFormat="Encase3" ; break;
+            case LIBEWF_FORMAT_ENCASE4: SubFormat="Encase4" ; break;
+            case LIBEWF_FORMAT_ENCASE5: SubFormat="Encase5" ; break;
+            case LIBEWF_FORMAT_ENCASE6: SubFormat="Encase6" ; break;
+            case LIBEWF_FORMAT_FTK    : SubFormat="FTK"     ; break;
+            case LIBEWF_FORMAT_SMART  : SubFormat="Smart"   ; break;
+            case LIBEWF_FORMAT_LVF    : SubFormat="LVF"     ; break;
+            case LIBEWF_FORMAT_LINEN5 : SubFormat="Linen5"  ; break;
+            case AEWF                 : SubFormat="Guymager"; break;
             default                   : CHK (ERROR_FILE_INVALID_EWF_FORMAT)
          }
          Str = QObject::tr("Expert Witness Format, sub-format %1") .arg(SubFormat);
          break;
-      case AFF:
+      case AAFF:
          Str = QObject::tr("Advanced forensic image");
          break;
       default:  CHK (ERROR_FILE_INVALID_FORMAT)
@@ -125,13 +126,14 @@ APIRET t_File::GetFormatExtension (t_Format Format, bool Clone, int DdSplitDecim
             case LIBEWF_FORMAT_ENCASE5:
             case LIBEWF_FORMAT_ENCASE6:
             case LIBEWF_FORMAT_LINEN5 :
-            case LIBEWF_FORMAT_FTK    : Wild=".E??"; Human=".Exx"; break;
+            case LIBEWF_FORMAT_FTK    :
+            case AEWF                 : Wild=".E??"; Human=".Exx"; break;
             case LIBEWF_FORMAT_SMART  : Wild=".s??"; Human=".sxx"; break;
             case LIBEWF_FORMAT_LVF    : Wild=".l??"; Human=".lxx"; break;
             default                   : CHK (ERROR_FILE_INVALID_EWF_FORMAT)
          }
          break;
-      case AFF:
+      case AAFF:
          Wild = ".aff";
          Human = Wild;
          break;
