@@ -362,6 +362,10 @@ Sind Sie mit diesen Änderungen einverstanden?</translation>
     </message>
     <message>
         <source>The split size for the configured EWF format must be in the range %1 - %2 MiB.For bigger files, switch to Encase6 format or later (see Guymager configuration file, parameter EwfFormat).</source>
+        <translation type="obsolete">Die Split-Grösse für das konfigurierte EWF-Format muss im Bereich %1 MiB - %2 MiB liegen. Grössere Dateien sind ab dem Format Encase6 möglich (siehe Guymager Konfigurationsdatei, Parameter EwfFormat).</translation>
+    </message>
+    <message>
+        <source>The split size for the configured EWF format must be in the range %1 - %2 MiB. For bigger files, switch to Encase6 format or later (see Guymager configuration file, parameter EwfFormat).</source>
         <translation>Die Split-Grösse für das konfigurierte EWF-Format muss im Bereich %1 MiB - %2 MiB liegen. Grössere Dateien sind ab dem Format Encase6 möglich (siehe Guymager Konfigurationsdatei, Parameter EwfFormat).</translation>
     </message>
 </context>
@@ -496,6 +500,10 @@ Guymager beendet sich automatisch in %1 Sekunden.</translation>
         <comment>Display that verification is off</comment>
         <translation>aus</translation>
     </message>
+    <message>
+        <source>Additional state info</source>
+        <translation>Status-Zusatzinformation</translation>
+    </message>
 </context>
 <context>
     <name>t_MainWindow</name>
@@ -560,7 +568,7 @@ Compilation timestamp: %2
 Compiled with gcc %3
 Using libewf %4
 Using libguytools %5</source>
-        <translation>GUYMAGER ist ein Linux-basierter, forensischer Imager
+        <translation type="obsolete">GUYMAGER ist ein Linux-basierter, forensischer Imager
 
 Version: %1
 Zeitstempel der Kompilation: %2
@@ -711,6 +719,34 @@ Benutzt libguytools %5</translation>
         <source>Exit after acquisitions completed</source>
         <translation>Guymager nach Abschluss aller Akquisitionen beenden</translation>
     </message>
+    <message>
+        <source>Aborted - Info file write error</source>
+        <translation>Abbruch - Schreibfehler Info-Datei</translation>
+    </message>
+    <message>
+        <source>Aborted - Acquisition start failure</source>
+        <translation>Abbruch - Startfehler Akquisition</translation>
+    </message>
+    <message>
+        <source>(not used as Guymager currently is configured to use its own EWF module)</source>
+        <translation>(inaktiv, da das Guymager-interne EWF-Modul konfiguriert ist)</translation>
+    </message>
+    <message>
+        <source>GUYMAGER is a Linux-based forensic imaging tool
+
+Version: %1
+Compilation timestamp: %2
+Compiled with gcc %3
+Linked with libewf %4 %5
+Linked with libguytools %6</source>
+        <translation>GUYMAGER ist ein Linux-basierter, forensischer Imager
+
+Version: %1
+Zeitstempel der Kompilation: %2
+Kompiliert mit gcc %3
+Benutzt libewf %4 %5
+Benutzt libguytools %6</translation>
+    </message>
 </context>
 <context>
     <name>t_Table</name>
@@ -1030,6 +1066,27 @@ Benutzt libguytools %5</translation>
         <source>Device used for cloning - cannot be acquired</source>
         <translation>Gerät durch Klon-Operation belegt - kann nicht akquiriert werden</translation>
     </message>
+    <message>
+        <source>Generated image files and their MD5 hashes</source>
+        <comment>Info file</comment>
+        <translation>Erezugte Image-Dateien und ihre MD5-Hashwerte</translation>
+    </message>
+    <message>
+        <source>No MD5 hashes available (the select EWF format does not support image file hashes)</source>
+        <translation>Keine MD5-Hashes verfügbar (das ausgewählte EWF-Format unterstützt das Hashen der Image-Dateien nicht)</translation>
+    </message>
+    <message>
+        <source>No MD5 hashes available (image verification checkbox was switched off)</source>
+        <translation>Keine MD5-Hashes verfügbar (das Häkchen zur Image-Verifikation war nicht gesetzt)</translation>
+    </message>
+    <message>
+        <source>No MD5 hashes available (configuration parameter CalcImageFileMD5 is off)</source>
+        <translation>Keine MD5-Hashes verfügbar (der Konfigurationsparameter CalcImageFileMD5 ist abgeschaltet)</translation>
+    </message>
+    <message>
+        <source>Cannot be acquired - see additional state info field</source>
+        <translation>Kann nicht akquiriert werden - siehe Status-Zusatzinformation</translation>
+    </message>
 </context>
 <context>
     <name>t_ThreadScan</name>
