@@ -344,7 +344,7 @@ Do you accept these changes?</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>The split size for the configured EWF format must be in the range %1 - %2 MiB.For bigger files, switch to Encase6 format or later (see Guymager configuration file, parameter EwfFormat).</source>
+        <source>The split size for the configured EWF format must be in the range %1 - %2 MiB. For bigger files, switch to Encase6 format or later (see Guymager configuration file, parameter EwfFormat).</source>
         <translation type="unfinished"></translation>
     </message>
 </context>
@@ -478,6 +478,10 @@ Guymager will exit automatically in %1 seconds.</source>
         <comment>Display that verification is off</comment>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Additional state info</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>t_MainWindow</name>
@@ -529,16 +533,6 @@ Guymager will exit automatically in %1 seconds.</source>
         <source>Debug information</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>GUYMAGER is a Linux-based forensic imaging tool
-
-Version: %1
-Compilation timestamp: %2
-Compiled with gcc %3
-Using libewf %4
-Using libguytools %5</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>About GUYMAGER</source>
         <comment>Dialog title</comment>
@@ -687,6 +681,28 @@ Using libguytools %5</source>
         <source>Exit after acquisitions completed</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Aborted - Info file write error</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Aborted - Acquisition start failure</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>(not used as Guymager currently is configured to use its own EWF module)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>GUYMAGER is a Linux-based forensic imaging tool
+
+Version: %1
+Compilation timestamp: %2
+Compiled with gcc %3
+Linked with libewf %4 %5
+Linked with libguytools %6</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>t_Table</name>
@@ -1006,6 +1022,27 @@ Using libguytools %5</source>
         <source>Device used for cloning - cannot be acquired</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Generated image files and their MD5 hashes</source>
+        <comment>Info file</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>No MD5 hashes available (the select EWF format does not support image file hashes)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>No MD5 hashes available (image verification checkbox was switched off)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>No MD5 hashes available (configuration parameter CalcImageFileMD5 is off)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Cannot be acquired - see additional state info field</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>t_ThreadScan</name>
