@@ -9,7 +9,7 @@
 //  Module:         Hash wrapper functions for uniform hash interface
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -54,7 +54,7 @@ APIRET HashMD5Init (t_pHashContextMD5 pContext)
    #ifdef USE_MD5_FROM_OPENSSL
       (void) MD5_Init (pContext);    // strange, man page says that MD5_Init returns void, but header file says it's int
    #else
-      md5_init (pContext);
+      MD5Init (pContext);
    #endif
 
    return NO_ERROR;
@@ -65,7 +65,7 @@ APIRET HashMD5Append (t_pHashContextMD5 pContext, const void *pData, int DataLen
    #ifdef USE_MD5_FROM_OPENSSL
       (void) MD5_Update (pContext, pData, (unsigned long) DataLen); // Same remark as for MD5_Init
    #else
-      md5_append (pContext, (md5_byte_t *)pData, DataLen);
+      MD5Append (pContext, pData, DataLen);
    #endif
 
    return NO_ERROR;
@@ -76,7 +76,7 @@ APIRET HashMD5Digest (t_pHashContextMD5 pContext, t_pHashMD5Digest pDigest)
    #ifdef USE_MD5_FROM_OPENSSL
       (void) MD5_Final (&pDigest->Buff[0], pContext); // Same remark as for MD5_Init
    #else
-      md5_finish (pContext, (md5_byte_t *)&pDigest->Buff[0]);
+      MD5Finish (pContext, &pDigest->Buff[0]);
    #endif
 
    return NO_ERROR;
@@ -110,7 +110,7 @@ APIRET HashSHA256Init (t_pHashContextSHA256 pContext)
    #ifdef USE_SHA256_FROM_OPENSSL
       (void) SHA256_Init (pContext);
    #else
-      sha256_starts (pContext);
+      SHA256Init (pContext);
    #endif
    return NO_ERROR;
 }
@@ -120,7 +120,7 @@ APIRET HashSHA256Append (t_pHashContextSHA256 pContext, void *pData, int DataLen
    #ifdef USE_SHA256_FROM_OPENSSL
       (void) SHA256_Update (pContext, pData, (unsigned long) DataLen);
    #else
-      sha256_update (pContext, (unsigned char *)pData, DataLen);
+      SHA256Append (pContext, (unsigned char *)pData, DataLen);
    #endif
    return NO_ERROR;
 }
@@ -130,7 +130,7 @@ APIRET HashSHA256Digest (t_pHashContextSHA256 pContext, t_pHashSHA256Digest pDig
    #ifdef USE_SHA256_FROM_OPENSSL
       (void) SHA256_Final (&pDigest->Buff[0], pContext);
    #else
-      sha256_finish (pContext, (unsigned char *)pDigest);
+      SHA256Finish (pContext, (unsigned char *)pDigest);
    #endif
    return NO_ERROR;
 }
