@@ -9,7 +9,7 @@
 //  Module:         The info field area
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -49,6 +49,8 @@ t_InfoField::t_InfoField ()
 t_InfoField::t_InfoField (QWidget *pParent)
    :QFrame (pParent)
 {
+   QString LabelParams;
+   
    CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_INFOFIELD_CONSTRUCTOR_NOT_SUPPORTED))
 
    setFrameStyle ((int)QFrame::Panel | (int)QFrame::Sunken);
@@ -60,16 +62,20 @@ t_InfoField::t_InfoField (QWidget *pParent)
    pLayout->addWidget (pOwn->pLabelParams);
    pLayout->addWidget (pOwn->pLabelValues);
    pLayout->addStretch();
-
-   pOwn->pLabelParams->setText (         tr("Size")
-                                + "\n" + tr("Sector size")
-                                + "\n" + tr("Image file")
-                                + "\n" + tr("Info file")
-                                + "\n" + tr("Current speed")
-                                + "\n" + tr("Started", "Start timestamp and running time")
-                                + "\n" + tr("Hash calculation")
-                                + "\n" + tr("Source verification")
-                                + "\n" + tr("Image verification"));
+   
+   if (CONFIG (CommandGetAddStateInfo)[0])
+      LabelParams = tr("Additional state info") + "\n";
+      
+   LabelParams +=       tr("Size")
+               + "\n" + tr("Sector size")
+               + "\n" + tr("Image file")
+               + "\n" + tr("Info file")
+               + "\n" + tr("Current speed")
+               + "\n" + tr("Started", "Start timestamp and running time")
+               + "\n" + tr("Hash calculation")
+               + "\n" + tr("Source verification")
+               + "\n" + tr("Image verification");
+   pOwn->pLabelParams->setText (LabelParams);
 }
 
 t_InfoField::~t_InfoField ()
@@ -86,10 +92,15 @@ void t_InfoField::SlotShowInfo (t_pDevice pDevice)
 
    if (pDevice)
    {
+      // Additional state info
+      // ---------------------
+      if (CONFIG (CommandGetAddStateInfo)[0])
+         StrValue = pDevice->AddStateInfo.Info + "\n";
+         
       // Size
       // ----
       Size = t_Device::GetSize (pDevice).toULongLong();
-      StrValue = MainGetpNumberLocale()->toString(Size) + " " + tr("bytes");
+      StrValue += MainGetpNumberLocale()->toString(Size) + " " + tr("bytes");
       StrValue += " ("  + t_Device::GetSizeHumanFrac (pDevice, false).toString();
       StrValue += " / " + t_Device::GetSizeHumanFrac (pDevice, true ).toString() + ")";
 
@@ -109,7 +120,7 @@ void t_InfoField::SlotShowInfo (t_pDevice pDevice)
       }
       else
       {
-         CHK_EXIT (t_File::GetFormatExtension   (pDevice->Acquisition.Format, pDevice->Acquisition.Clone, NULL, &Format))
+         CHK_EXIT (t_File::GetFormatExtension   (pDevice->Acquisition.Format, pDevice->Acquisition.Clone, 0, NULL, &Format))
          StrValue += "\n" + pDevice->Acquisition.ImagePath + pDevice->Acquisition.ImageFilename + QSTR_TO_PSZ(Format);
          StrValue += "\n" + pDevice->Acquisition.InfoPath  + pDevice->Acquisition.InfoFilename  + t_File::pExtensionInfo;
       }
