@@ -33,7 +33,7 @@
 #include <signal.h>
 #include <locale.h>
 #include <pwd.h>
-#include "proc/sysinfo.h"  // Required in order to get the amount of memory installed
+// #include "proc/sysinfo.h"  // Required in order to get the amount of memory installed
 
 #include <QApplication>
 #include <QtGui>
@@ -274,8 +274,10 @@ static APIRET MainGo (int argc, char *argv[], bool *pAutoExit)
    pLibEwfVersionInstalled = (const char *) libewf_get_version();
    LOG_INFO ("Libguytools version installed on this PC: %s", pLibGuyToolsVersionInstalled)
    LOG_INFO ("Libewf      version installed on this PC: %s", pLibEwfVersionInstalled)
-   meminfo();
-   LOG_INFO ("Total amount of memory installed: %lu KB", kb_main_total)
+
+
+   unsigned long long Bytes = UtilGetInstalledRAM ();
+   LOG_INFO ("Total amount of memory installed: %0.1f MB", Bytes/(1024.0*1024.0))
    
 //   pLibEwfVersionCompiled = LIBEWF_VERSION_STRING;
 //
