@@ -10,7 +10,7 @@
 //                  widget
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -44,6 +44,10 @@
 #include "table.h"
 #include "dlgmessage.h"
 #include "dlgautoexit.h"
+#include "threadread.h"
+#include "threadhash.h"
+#include "threadcompress.h"
+#include "threadwrite.h"
 
 
 // -----------------------------
@@ -83,6 +87,7 @@ t_DeviceListModel::t_DeviceListModel (t_pDeviceList pDeviceList)
       COL_ASSOC (tr("Linux\ndevice"          , "Column of device table"), t_Device::GetLinuxDevice   , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
       COL_ASSOC (tr("Model"                  , "Column of device table"), t_Device::GetModel         , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
       COL_ASSOC (tr("State"                  , "Column of device table"), t_Device::GetState         , t_ItemDelegate::DISPLAYTYPE_STATE   , LEFT  , 200)
+//    COL_ASSOC (tr("Additional\nstate info" , "Column of device table"), t_Device::GetAddStateInfo  , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
       COL_ASSOC (tr("Size"                   , "Column of device table"), t_Device::GetSizeHuman     , t_ItemDelegate::DISPLAYTYPE_STANDARD, RIGHT ,   0)
       COL_ASSOC (tr("Hidden\nAreas"          , "Column of device table"), t_Device::GetHiddenAreas   , t_ItemDelegate::DISPLAYTYPE_STANDARD, RIGHT ,   0)
       COL_ASSOC (tr("Bad\nsectors"           , "Column of device table"), t_Device::GetBadSectorCount, t_ItemDelegate::DISPLAYTYPE_STANDARD, RIGHT ,   0)
@@ -90,9 +95,9 @@ t_DeviceListModel::t_DeviceListModel (t_pDeviceList pDeviceList)
       COL_ASSOC (tr("Average\nSpeed\n[MB/s]" , "Column of device table"), t_Device::GetAverageSpeed  , t_ItemDelegate::DISPLAYTYPE_STANDARD, RIGHT ,   0)
       COL_ASSOC (tr("Time\nremaining"        , "Column of device table"), t_Device::GetRemaining     , t_ItemDelegate::DISPLAYTYPE_STANDARD, CENTER,   0)
       COL_ASSOC (tr("FIFO queues\nusage\n[%]", "Column of device table"), t_Device::GetFifoStatus    , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
-//    COL_ASSOC (tr("Sector\nsize\nlog."     , "Column of device table"), t_Device::GetSectorSize    , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT, 0      )
-//    COL_ASSOC (tr("Sector\nsize\nphys."    , "Column of device table"), t_Device::GetSectorSizePhys, t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT, 0      )
-//    COL_ASSOC (tr("Current\nSpeed\n[MB/s]" , "Column of device table"), t_Device::GetCurrentSpeed  , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT, 0      )
+//    COL_ASSOC (tr("Sector\nsize\nlog."     , "Column of device table"), t_Device::GetSectorSize    , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
+//    COL_ASSOC (tr("Sector\nsize\nphys."    , "Column of device table"), t_Device::GetSectorSizePhys, t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
+//    COL_ASSOC (tr("Current\nSpeed\n[MB/s]" , "Column of device table"), t_Device::GetCurrentSpeed  , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
    #undef COL_ASSOC
    #undef LEFT
    #undef RIGHT
@@ -157,7 +162,14 @@ QVariant t_DeviceListModel::data(const QModelIndex &Index, int Role) const
             pDev = poDeviceList->at(Index.row());
             if (pDev->Local)
                  Value = QBrush (CONFIG_COLOR(COLOR_LOCALDEVICES));
-            else Value = QVariant();
+            else switch (pDev->AddStateInfo.Color)
+                 {
+                    case 1 : Value = QBrush (CONFIG_COLOR(COLOR_ADDITIONALSTATE1)); break;
+                    case 2 : Value = QBrush (CONFIG_COLOR(COLOR_ADDITIONALSTATE2)); break;
+                    case 3 : Value = QBrush (CONFIG_COLOR(COLOR_ADDITIONALSTATE3)); break;
+                    case 4 : Value = QBrush (CONFIG_COLOR(COLOR_ADDITIONALSTATE4)); break;
+                    default: Value = QVariant();
+                 }
             break;
          case Qt::DisplayRole:
             if (Index.column() >= ColAssocList.count())
@@ -279,6 +291,7 @@ t_MainWindow::t_MainWindow (t_pDeviceList pDeviceList, QWidget *pParent, Qt::WFl
 
    CHK_EXIT (CreateMenu ())
    setWindowTitle ("GUYMAGER");
+//   setSizeGripEnabled (true);
 
    // Create models and view according do the following layering:
    //    TableView        -> GUI
@@ -424,6 +437,7 @@ void t_MainWindow::SlotScanFinished (t_pDeviceList pNewDeviceList)
 {
    t_pDeviceList pDeviceList;
    t_pDevice     pDev, pNewDev;
+   bool           NewDevice;
    int            i;
 
 //   LOG_INFO ("%d devices found", pNewDeviceList->count())
@@ -433,7 +447,8 @@ void t_MainWindow::SlotScanFinished (t_pDeviceList pNewDeviceList)
       pDev = pDeviceList->at (i);
       if (pDev->SpecialDevice)
            pDev->Checked = true;   // Checked is used to remember which devices we have seen and which ones not. Treat
-      else pDev->Checked = false;  // special devices as already seen, thus, they are going to be removed from the list.
+      else pDev->Checked = false;  // special devices as already seen, thus, they are not going to be removed from the list.
+      pDev->AddedNow=false;
    }
 
    for (i=0; i<pNewDeviceList->count(); i++)
@@ -451,14 +466,19 @@ void t_MainWindow::SlotScanFinished (t_pDeviceList pNewDeviceList)
 //         LOG_INFO ("No match for device %s %s %s %llu", QSTR_TO_PSZ (pNewDev->LinuxDevice), QSTR_TO_PSZ (pNewDev->Model), QSTR_TO_PSZ (pNewDev->SerialNumber), pNewDev->Size)
 //      }
 
-      if (pDev)
+      if (pDev)                           // Normally, if pDev has been found, it means that pNewDev already is known and should not be added another time. However,
+           NewDevice = (pDev->AddedNow);  // function t_DeviceList::MatchDevice might be wrong in some cases and finds devices it shouldn't. This might happen
+      else NewDevice = true;              // for devices of the same size and not having serial numbers. In order to prevent from these cases, we say that one device scan
+                                          // very unlikely returns 2 entries for the same device (nobody unplugs and replugs a device that fast). So, if t_DeviceList::MatchDevice
+                                          // returns a match with a device that has been added while we are processing this loop, we simply ignore the match and add the device anyway.
+      if (!NewDevice)
       {
          pDev->Checked = true;
          if ((pDev->State == t_Device::AcquirePaused) ||
              (pDev->State == t_Device::VerifyPaused))
          {
             LOG_INFO ("Switching %s to connected again", QSTR_TO_PSZ(pNewDev->LinuxDevice))
-            pDev->LinuxDevice = pNewDev->LinuxDevice;    // Linux may choose a new path when reconnecting the device
+               pDev->LinuxDevice = pNewDev->LinuxDevice;    // Linux may choose a new path when reconnecting the device
             if (pDev->State == t_Device::AcquirePaused)  // (1)
                  pDev->State = t_Device::Acquire;
             else pDev->State = t_Device::Verify;
@@ -466,7 +486,8 @@ void t_MainWindow::SlotScanFinished (t_pDeviceList pNewDeviceList)
       }
       else
       {
-         pNewDev->Checked = true;
+         pNewDev->Checked  = true;
+         pNewDev->AddedNow = true;
          pNewDeviceList->removeAt (i--);                 // (3)
          pDeviceList->append (pNewDev);
       }
@@ -536,11 +557,14 @@ void t_MainWindow::closeEvent (QCloseEvent *pEvent)
 
 void t_MainWindow::SlotDebug (void)
 {
-   QString DebugInfo;
-   quint64 FifoAllocs, FifoFrees;
-   qint64  FifoRemaining;
-   qint64  FifoAllocated;
+   t_pDevice pDevice;
+   QString    DebugInfo;
+   quint64    FifoAllocs, FifoFrees;
+   qint64     FifoRemaining;
+   qint64     FifoAllocated;
 
+   // FIFO memory statistics
+   // ----------------------
    CHK_EXIT (FifoGetStatistics (FifoAllocs, FifoFrees, FifoAllocated))
    FifoRemaining = FifoAllocs - FifoFrees;
    DebugInfo  = "FIFO Buffers";
@@ -549,6 +573,22 @@ void t_MainWindow::SlotDebug (void)
    DebugInfo += QString ("\n   %1 remaining") .arg(FifoRemaining, 10);
    DebugInfo += QString ("\n   %1 MB"       ) .arg((double)FifoAllocated / (1024.0*1024.0), 10, 'f', 1);
 
+   // Thread messages
+   // ---------------
+   #define ADD_MESSAGE(pThread, Name) if (pDevice->pThread) DebugInfo += QString ("\n   %1: %2") .arg(Name, -13) .arg(pDevice->pThread->GetDebugMessage());
+
+   for (int i=0; i<pOwn->pDeviceList->count(); i++)
+   {
+      pDevice = pOwn->pDeviceList->at(i);
+      DebugInfo += QString ("\nThread messages for %1") .arg(pDevice->LinuxDevice);
+      ADD_MESSAGE (pThreadRead, "Read")
+      ADD_MESSAGE (pThreadHash, "Hash")
+      for (int j=0; j<pDevice->ThreadCompressList.count(); j++)
+         ADD_MESSAGE (ThreadCompressList.at(j), QString("Compress %1").arg(j))
+      ADD_MESSAGE (pThreadWrite, "Write")
+   }
+   DebugInfo += QString ("\nThreadScan: %1") .arg(pOwn->pThreadScan->GetDebugMessage());
+
    CHK_EXIT (t_DlgMessage::Show (tr("Debug information"), DebugInfo, true))
 }
 
@@ -556,22 +596,26 @@ void t_MainWindow::SlotAboutGuymager (void)
 {
    const char *pLibGuyToolsVersionInstalled;
    const char *pLibEwfVersionInstalled;
+   QString      EwfRemark;
 
    t_Log::GetLibGuyToolsVersion (&pLibGuyToolsVersionInstalled);
 
    pLibEwfVersionInstalled = (const char *) libewf_get_version ();
 
+   if (CONFIG(EwfFormat) == t_File::AEWF)
+        EwfRemark = tr("(not used as Guymager currently is configured to use its own EWF module)");
+   else EwfRemark = "";
    QMessageBox::about(this, tr("About GUYMAGER", "Dialog title"),
                             tr("GUYMAGER is a Linux-based forensic imaging tool\n\n"
                                "Version: %1\n"
                                "Compilation timestamp: %2\n"
                                "Compiled with gcc %3\n"
-                               "Using libewf %4\n"
-                               "Using libguytools %5")
+                               "Linked with libewf %4 %5\n"
+                               "Linked with libguytools %6")
                                .arg(pCompileInfoVersion)
                                .arg(pCompileInfoTimestamp)
                                .arg(__VERSION__)
-                               .arg(pLibEwfVersionInstalled)
+                               .arg(pLibEwfVersionInstalled) .arg(EwfRemark)
                                .arg(pLibGuyToolsVersionInstalled));
 }
 
