@@ -11,6 +11,23 @@
 //                  copied from Mark Lord's famous hdparm.
 // ****************************************************************************
 
+// Copyright 2008, 2009, 2010, 2011, 2012 Guy Voncken
+//
+// This file is part of guymager.
+//
+// guymager is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+//
+// guymager is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with guymager. If not, see <http://www.gnu.org/licenses/>.
+
 #include <stdlib.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -754,6 +771,7 @@ APIRET t_MediaInfo::QueryDevice (const char *pLinuxDevice)
    InitUnknown();
 
    pMediaDriveCmdATA   = (t_pMediaDriveCmd) malloc (sizeof(t_MediaDriveCmd));
+   memset (pMediaDriveCmdATA, 0, sizeof(t_MediaDriveCmd));
 
    File = open (pLinuxDevice, O_RDONLY);
    if (File < 0)
