@@ -1,45 +1,61 @@
-// sha256 calculation on i386 and amd64 processors.
-/*
- *  FIPS-180-2 compliant SHA-256 implementation
- *
- *  Copyright (C) 2001-2003  Christophe Devine
- *  Copyright (C) 2010,2011  Guy Voncken <vogu00@gmail.com>
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// ****************************************************************************
+//  Project:        GUYMAGER
+// ****************************************************************************
+//  Programmer:     Guy Voncken
+//                  Police Grand-Ducale
+//                  Service de Police Judiciaire
+//                  Section Nouvelles Technologies
+// ****************************************************************************
+//  Module:         MD5 calculation
+// ****************************************************************************
 
-// This file has initially been downloaded from
-//     www.spale.com/download/scrypt/scrypt1.0/sha256.c
+// Copyright 2008, 2009, 2010, 2011, 2012 Guy Voncken
 //
-// The following optimisation and debugging work has been done by Guy Voncken:
-//     - The CPU intensive parts have been rewritten in assembler for better
-//       performance on i386 and amd64 machines.
-//     - The code has been corrected for big endian machines. The corrections
-//       have been tested on a PowerPC machine.
+// This file is part of guymager.
+//
+// guymager is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+//
+// guymager is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with guymager. If not, see <http://www.gnu.org/licenses/>.
 
 
+// This module was initially based on the file
+//    www.spale.com/download/scrypt/scrypt1.0/sha256.c
+// copyrighted by Christophe Devine, with assembler i386/amd64-optimisation
+// and big endian debugging added by Guy Voncken. That optimised code
+// was faster than the Linux' standard tool sha256sum. You find it in
+// this module under SHA256_OLD macro.
+//
+// In later tests it was observed that sha256sum (or gcc) must have been
+// improved; sha256sum was the fastet now. That's why this module nowadays
+// is based on sha256sum's code (see Debian package coreutils, for example).
+// The core functions have been mainly copy/pasted; they have been written
+// by David Madore and Scott G. Miller and are copyrighted by the Free
+// Software Foundation, Inc.
+
+
+//#define SHA256_OLD
+
+#include <stddef.h>
+#include "sha256.h"
+
+#ifdef SHA256_OLD
+
 #include <string.h>
 #include <netinet/in.h>
 
-#include "sha256.h"
-
 #if defined(__i386__) || defined (__amd64__)
    #define SHA256_USE_OPTIMISED_ASSEMBLER_CODE
 #endif
 
-
 // Guy: The original GET_UINT32 and PUT_UINT32 macros are
 //   - too slow
 //   - not working on powerpc
@@ -69,24 +85,22 @@
 #define PUT_UINT32(n,b,i)                       \
    *((uint32 *)&(b)[i]) = ntohl((n));
 
-void sha256_starts( sha256_context *ctx )
+void SHA256Init (t_pSHA256Context pContext)
 {
-    ctx->total[0] = 0;
-    ctx->total[1] = 0;
+    pContext->total[0] = 0;
+    pContext->total[1] = 0;
 
-    ctx->state[0] = 0x6A09E667;
-    ctx->state[1] = 0xBB67AE85;
-    ctx->state[2] = 0x3C6EF372;
-    ctx->state[3] = 0xA54FF53A;
-    ctx->state[4] = 0x510E527F;
-    ctx->state[5] = 0x9B05688C;
-    ctx->state[6] = 0x1F83D9AB;
-    ctx->state[7] = 0x5BE0CD19;
+    pContext->state[0] = 0x6A09E667;
+    pContext->state[1] = 0xBB67AE85;
+    pContext->state[2] = 0x3C6EF372;
+    pContext->state[3] = 0xA54FF53A;
+    pContext->state[4] = 0x510E527F;
+    pContext->state[5] = 0x9B05688C;
+    pContext->state[6] = 0x1F83D9AB;
+    pContext->state[7] = 0x5BE0CD19;
 }
 
-
 #ifdef SHA256_USE_OPTIMISED_ASSEMBLER_CODE
-
    #define P(a,b,c,d,e,f,g,h,x,K)                                                                       \
        __asm__  __volatile__ (                                                                          \
                 /* ------------------------------------------------------------------------------- */   \
@@ -145,7 +159,6 @@ void sha256_starts( sha256_context *ctx )
                                                                                                 \
             :"%eax", "%ebx", "%ecx", "%edx", "%cc", "memory")
 
-
    #if defined(__i386__)
       #define PR(a,b,c,d,e,f,g,h,i,K)                                         \
           __asm__  __volatile__ (                                             \
@@ -235,9 +248,7 @@ void sha256_starts( sha256_context *ctx )
                /*   2       3       4       5       6       7       8       9       10       0       1 */   \
                                                                                                             \
                :"%eax", "%ebx", "%ecx", "%edx", "%cc", "memory")
-
    #elif defined(__amd64__)
-
       #define PR(a,b,c,d,e,f,g,h,i,K)                                         \
           __asm__  __volatile__ (                                             \
                                                                               \
@@ -359,10 +370,9 @@ void sha256_starts( sha256_context *ctx )
    #define PR(a,b,c,d,e,f,g,h,i,K)                 \
       P(a,b,c,d,e,f,g,h,R(i),K)
 
-
 #endif
 
-void sha256_process( sha256_context *ctx, uint8 data[64] )
+static void SHA256Process(t_pSHA256Context pContext, uint8 data[64] )
 {
     #ifndef SHA256_USE_OPTIMISED_ASSEMBLER_CODE
        uint32 temp1, temp2;
@@ -388,14 +398,14 @@ void sha256_process( sha256_context *ctx, uint8 data[64] )
     GET_UINT32( W[14], data, 56 );
     GET_UINT32( W[15], data, 60 );
 
-    A = ctx->state[0];
-    B = ctx->state[1];
-    C = ctx->state[2];
-    D = ctx->state[3];
-    E = ctx->state[4];
-    F = ctx->state[5];
-    G = ctx->state[6];
-    H = ctx->state[7];
+    A = pContext->state[0];
+    B = pContext->state[1];
+    C = pContext->state[2];
+    D = pContext->state[3];
+    E = pContext->state[4];
+    F = pContext->state[5];
+    G = pContext->state[6];
+    H = pContext->state[7];
 
     P( A, B, C, D, E, F, G, H, W[ 0],  0x428A2F98 );
     P( H, A, B, C, D, E, F, G, W[ 1],  0x71374491 );
@@ -462,36 +472,36 @@ void sha256_process( sha256_context *ctx, uint8 data[64] )
     PR(C, D, E, F, G, H, A, B,   62 , -0x41065C09 );
     PR(B, C, D, E, F, G, H, A,   63 , -0x398E870E );
 
-    ctx->state[0] += A;
-    ctx->state[1] += B;
-    ctx->state[2] += C;
-    ctx->state[3] += D;
-    ctx->state[4] += E;
-    ctx->state[5] += F;
-    ctx->state[6] += G;
-    ctx->state[7] += H;
+    pContext->state[0] += A;
+    pContext->state[1] += B;
+    pContext->state[2] += C;
+    pContext->state[3] += D;
+    pContext->state[4] += E;
+    pContext->state[5] += F;
+    pContext->state[6] += G;
+    pContext->state[7] += H;
 }
 
-void sha256_update( sha256_context *ctx, uint8 *input, uint32 length )
+void SHA256Append (t_pSHA256Context pContext, uint8 *input, uint32 length )
 {
     uint32 left, fill;
 
     if( ! length ) return;
 
-    left = ctx->total[0] & 0x3F;
+    left = pContext->total[0] & 0x3F;
     fill = 64 - left;
 
-    ctx->total[0] += length;
-    ctx->total[0] &= 0xFFFFFFFF;
+    pContext->total[0] += length;
+    pContext->total[0] &= 0xFFFFFFFF;
 
-    if( ctx->total[0] < length )
-        ctx->total[1]++;
+    if( pContext->total[0] < length )
+        pContext->total[1]++;
 
     if( left && length >= fill )
     {
-        memcpy( (void *) (ctx->buffer + left),
+        memcpy( (void *) (pContext->buffer + left),
                 (void *) input, fill );
-        sha256_process( ctx, ctx->buffer );
+        SHA256Process (pContext, pContext->buffer);
         length -= fill;
         input  += fill;
         left = 0;
@@ -499,19 +509,19 @@ void sha256_update( sha256_context *ctx, uint8 *input, uint32 length )
 
     while( length >= 64 )
     {
-        sha256_process( ctx, input );
+        SHA256Process (pContext, input );
         length -= 64;
         input  += 64;
     }
 
     if( length )
     {
-        memcpy( (void *) (ctx->buffer + left),
+        memcpy( (void *) (pContext->buffer + left),
                 (void *) input, length );
     }
 }
 
-static uint8 sha256_padding[64] =
+static uint8 SHA256Padding[64] =
 {
  0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -519,35 +529,329 @@ static uint8 sha256_padding[64] =
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
 };
 
-void sha256_finish( sha256_context *ctx, uint8 digest[32] )
+void SHA256Finish (t_pSHA256Context pContext, uint8 digest[32])
 {
     uint32 last, padn;
     uint32 high, low;
     uint8 msglen[8];
 
-    high = ( ctx->total[0] >> 29 )
-         | ( ctx->total[1] <<  3 );
-    low  = ( ctx->total[0] <<  3 );
+    high = ( pContext->total[0] >> 29 )
+         | ( pContext->total[1] <<  3 );
+    low  = ( pContext->total[0] <<  3 );
 
     PUT_UINT32( high, msglen, 0 );
     PUT_UINT32( low,  msglen, 4 );
 
-    last = ctx->total[0] & 0x3F;
+    last = pContext->total[0] & 0x3F;
     padn = ( last < 56 ) ? ( 56 - last ) : ( 120 - last );
 
-    sha256_update( ctx, sha256_padding, padn );
-    sha256_update( ctx, msglen, 8 );
+    SHA256Append (pContext, SHA256Padding, padn);
+    SHA256Append (pContext, msglen, 8);
 
-    PUT_UINT32( ctx->state[0], digest,  0 );
-    PUT_UINT32( ctx->state[1], digest,  4 );
-    PUT_UINT32( ctx->state[2], digest,  8 );
-    PUT_UINT32( ctx->state[3], digest, 12 );
-    PUT_UINT32( ctx->state[4], digest, 16 );
-    PUT_UINT32( ctx->state[5], digest, 20 );
-    PUT_UINT32( ctx->state[6], digest, 24 );
-    PUT_UINT32( ctx->state[7], digest, 28 );
+    PUT_UINT32(pContext->state[0], digest,  0);
+    PUT_UINT32(pContext->state[1], digest,  4);
+    PUT_UINT32(pContext->state[2], digest,  8);
+    PUT_UINT32(pContext->state[3], digest, 12);
+    PUT_UINT32(pContext->state[4], digest, 16);
+    PUT_UINT32(pContext->state[5], digest, 20);
+    PUT_UINT32(pContext->state[6], digest, 24);
+    PUT_UINT32(pContext->state[7], digest, 28);
 }
 
+#else  // SHA256_OLD
+
+#include <stddef.h>
+#include <stdlib.h>
+#include <string.h>
+
+#ifdef WORDS_BIGENDIAN
+   #define SWAP(n) (n)
+#else
+   #define SWAP(n) (((n) << 24) | (((n) & 0xff00) << 8) | (((n) >> 8) & 0xff00) | ((n) >> 24))
+#endif
+
+#define BLOCKSIZE 32768
+#if BLOCKSIZE % 64 != 0
+  #error "invalid BLOCKSIZE"
+#endif
+
+static const unsigned char SHA256PadBuf[64] = { 0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                                                   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                                                   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                                                   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+
+void SHA256Init (t_pSHA256Context pContext)
+{
+   pContext->state[0] = 0x6a09e667UL;
+   pContext->state[1] = 0xbb67ae85UL;
+   pContext->state[2] = 0x3c6ef372UL;
+   pContext->state[3] = 0xa54ff53aUL;
+   pContext->state[4] = 0x510e527fUL;
+   pContext->state[5] = 0x9b05688cUL;
+   pContext->state[6] = 0x1f83d9abUL;
+   pContext->state[7] = 0x5be0cd19UL;
+
+   pContext->total[0] = pContext->total[1] = 0;
+   pContext->buflen   = 0;
+}
+
+static inline void set_uint32 (char *cp, uint32 v)
+{
+   memcpy (cp, &v, sizeof v);
+}
+
+// SHA256 round constants
+#define K(I) sha256_round_constants[I]
+static const uint32 sha256_round_constants[64] =
+{
+   0x428a2f98UL, 0x71374491UL, 0xb5c0fbcfUL, 0xe9b5dba5UL,
+   0x3956c25bUL, 0x59f111f1UL, 0x923f82a4UL, 0xab1c5ed5UL,
+   0xd807aa98UL, 0x12835b01UL, 0x243185beUL, 0x550c7dc3UL,
+   0x72be5d74UL, 0x80deb1feUL, 0x9bdc06a7UL, 0xc19bf174UL,
+   0xe49b69c1UL, 0xefbe4786UL, 0x0fc19dc6UL, 0x240ca1ccUL,
+   0x2de92c6fUL, 0x4a7484aaUL, 0x5cb0a9dcUL, 0x76f988daUL,
+   0x983e5152UL, 0xa831c66dUL, 0xb00327c8UL, 0xbf597fc7UL,
+   0xc6e00bf3UL, 0xd5a79147UL, 0x06ca6351UL, 0x14292967UL,
+   0x27b70a85UL, 0x2e1b2138UL, 0x4d2c6dfcUL, 0x53380d13UL,
+   0x650a7354UL, 0x766a0abbUL, 0x81c2c92eUL, 0x92722c85UL,
+   0xa2bfe8a1UL, 0xa81a664bUL, 0xc24b8b70UL, 0xc76c51a3UL,
+   0xd192e819UL, 0xd6990624UL, 0xf40e3585UL, 0x106aa070UL,
+   0x19a4c116UL, 0x1e376c08UL, 0x2748774cUL, 0x34b0bcb5UL,
+   0x391c0cb3UL, 0x4ed8aa4aUL, 0x5b9cca4fUL, 0x682e6ff3UL,
+   0x748f82eeUL, 0x78a5636fUL, 0x84c87814UL, 0x8cc70208UL,
+   0x90befffaUL, 0xa4506cebUL, 0xbef9a3f7UL, 0xc67178f2UL,
+};
+
+// Round functions
+#define F2(A,B,C) ( ( A & B ) | ( C & ( A | B ) ) )
+#define F1(E,F,G) ( G ^ ( E & ( F ^ G ) ) )
+
+static void SHA256ProcessBlock (t_pSHA256Context pContext, const void *buffer, size_t len) __attribute__((optimize("-O3")));
+static void SHA256ProcessBlock (t_pSHA256Context pContext, const void *buffer, size_t len)
+{
+   size_t        nwords = len / sizeof (uint32);
+   const uint32 *words  = (uint32 *)buffer;
+   const uint32 *endp   = words + nwords;
+   uint32 a = pContext->state[0];
+   uint32 b = pContext->state[1];
+   uint32 c = pContext->state[2];
+   uint32 d = pContext->state[3];
+   uint32 e = pContext->state[4];
+   uint32 f = pContext->state[5];
+   uint32 g = pContext->state[6];
+   uint32 h = pContext->state[7];
+   uint32 x[16];
+
+   pContext->total[0] += len;
+   if (pContext->total[0] < len)
+      ++pContext->total[1];
+
+   #define rol(x, n) (((x) << (n)) | ((x) >> (32 - (n))))
+   #define S0(x)     (rol(x,25) ^ rol(x,14) ^ (x>> 3))
+   #define S1(x)     (rol(x,15) ^ rol(x,13) ^ (x>>10))
+   #define SS0(x)    (rol(x,30) ^ rol(x,19) ^ rol(x,10))
+   #define SS1(x)    (rol(x,26) ^ rol(x,21) ^ rol(x, 7))
+
+   #define M(I) ( tm =   S1(x[(I- 2)&0x0f]) + x[(I-7)&0x0f] \
+                       + S0(x[(I-15)&0x0f]) + x[ I   &0x0f] \
+                  , x[I&0x0f] = tm )
+
+   #define R(A,B,C,D,E,F,G,H,K,M)  \
+      do                           \
+      {                            \
+         t0  = SS0(A) + F2(A,B,C); \
+         t1  = H + SS1(E)          \
+             + F1(E,F,G)           \
+             + K                   \
+             + M;                  \
+          D += t1;                 \
+          H  = t0 + t1;            \
+      } while(0)
+
+   while (words < endp)
+   {
+      uint32 tm;
+      uint32 t0, t1;
+      int t;
+
+      for (t = 0; t < 16; t++)
+      {
+         x[t] = SWAP (*words);
+         words++;
+      }
+
+      R( a, b, c, d, e, f, g, h, K( 0), x[ 0] );
+      R( h, a, b, c, d, e, f, g, K( 1), x[ 1] );
+      R( g, h, a, b, c, d, e, f, K( 2), x[ 2] );
+      R( f, g, h, a, b, c, d, e, K( 3), x[ 3] );
+      R( e, f, g, h, a, b, c, d, K( 4), x[ 4] );
+      R( d, e, f, g, h, a, b, c, K( 5), x[ 5] );
+      R( c, d, e, f, g, h, a, b, K( 6), x[ 6] );
+      R( b, c, d, e, f, g, h, a, K( 7), x[ 7] );
+      R( a, b, c, d, e, f, g, h, K( 8), x[ 8] );
+      R( h, a, b, c, d, e, f, g, K( 9), x[ 9] );
+      R( g, h, a, b, c, d, e, f, K(10), x[10] );
+      R( f, g, h, a, b, c, d, e, K(11), x[11] );
+      R( e, f, g, h, a, b, c, d, K(12), x[12] );
+      R( d, e, f, g, h, a, b, c, K(13), x[13] );
+      R( c, d, e, f, g, h, a, b, K(14), x[14] );
+      R( b, c, d, e, f, g, h, a, K(15), x[15] );
+      R( a, b, c, d, e, f, g, h, K(16), M(16) );
+      R( h, a, b, c, d, e, f, g, K(17), M(17) );
+      R( g, h, a, b, c, d, e, f, K(18), M(18) );
+      R( f, g, h, a, b, c, d, e, K(19), M(19) );
+      R( e, f, g, h, a, b, c, d, K(20), M(20) );
+      R( d, e, f, g, h, a, b, c, K(21), M(21) );
+      R( c, d, e, f, g, h, a, b, K(22), M(22) );
+      R( b, c, d, e, f, g, h, a, K(23), M(23) );
+      R( a, b, c, d, e, f, g, h, K(24), M(24) );
+      R( h, a, b, c, d, e, f, g, K(25), M(25) );
+      R( g, h, a, b, c, d, e, f, K(26), M(26) );
+      R( f, g, h, a, b, c, d, e, K(27), M(27) );
+      R( e, f, g, h, a, b, c, d, K(28), M(28) );
+      R( d, e, f, g, h, a, b, c, K(29), M(29) );
+      R( c, d, e, f, g, h, a, b, K(30), M(30) );
+      R( b, c, d, e, f, g, h, a, K(31), M(31) );
+      R( a, b, c, d, e, f, g, h, K(32), M(32) );
+      R( h, a, b, c, d, e, f, g, K(33), M(33) );
+      R( g, h, a, b, c, d, e, f, K(34), M(34) );
+      R( f, g, h, a, b, c, d, e, K(35), M(35) );
+      R( e, f, g, h, a, b, c, d, K(36), M(36) );
+      R( d, e, f, g, h, a, b, c, K(37), M(37) );
+      R( c, d, e, f, g, h, a, b, K(38), M(38) );
+      R( b, c, d, e, f, g, h, a, K(39), M(39) );
+      R( a, b, c, d, e, f, g, h, K(40), M(40) );
+      R( h, a, b, c, d, e, f, g, K(41), M(41) );
+      R( g, h, a, b, c, d, e, f, K(42), M(42) );
+      R( f, g, h, a, b, c, d, e, K(43), M(43) );
+      R( e, f, g, h, a, b, c, d, K(44), M(44) );
+      R( d, e, f, g, h, a, b, c, K(45), M(45) );
+      R( c, d, e, f, g, h, a, b, K(46), M(46) );
+      R( b, c, d, e, f, g, h, a, K(47), M(47) );
+      R( a, b, c, d, e, f, g, h, K(48), M(48) );
+      R( h, a, b, c, d, e, f, g, K(49), M(49) );
+      R( g, h, a, b, c, d, e, f, K(50), M(50) );
+      R( f, g, h, a, b, c, d, e, K(51), M(51) );
+      R( e, f, g, h, a, b, c, d, K(52), M(52) );
+      R( d, e, f, g, h, a, b, c, K(53), M(53) );
+      R( c, d, e, f, g, h, a, b, K(54), M(54) );
+      R( b, c, d, e, f, g, h, a, K(55), M(55) );
+      R( a, b, c, d, e, f, g, h, K(56), M(56) );
+      R( h, a, b, c, d, e, f, g, K(57), M(57) );
+      R( g, h, a, b, c, d, e, f, K(58), M(58) );
+      R( f, g, h, a, b, c, d, e, K(59), M(59) );
+      R( e, f, g, h, a, b, c, d, K(60), M(60) );
+      R( d, e, f, g, h, a, b, c, K(61), M(61) );
+      R( c, d, e, f, g, h, a, b, K(62), M(62) );
+      R( b, c, d, e, f, g, h, a, K(63), M(63) );
+
+      a = pContext->state[0] += a;
+      b = pContext->state[1] += b;
+      c = pContext->state[2] += c;
+      d = pContext->state[3] += d;
+      e = pContext->state[4] += e;
+      f = pContext->state[5] += f;
+      g = pContext->state[6] += g;
+      h = pContext->state[7] += h;
+   }
+}
+
+void SHA256Finish (t_pSHA256Context pContext, void *pDigest)
+{
+   // Take yet unprocessed bytes into account
+   size_t bytes = pContext->buflen;
+   size_t size = (bytes < 56) ? 64 / 4 : 64 * 2 / 4;
+
+   // Count remaining bytes
+   pContext->total[0] += bytes;
+   if (pContext->total[0] < bytes)
+      ++pContext->total[1];
+
+   set_uint32 ((char *) &pContext->buffer[size - 2], SWAP ((pContext->total[1] << 3) | (pContext->total[0] >> 29)));
+   set_uint32 ((char *) &pContext->buffer[size - 1], SWAP (pContext->total[0] << 3));
+
+   memcpy (&((char *) pContext->buffer)[bytes], SHA256PadBuf, (size - 2) * 4 - bytes);
+
+   // Process last bytes
+   SHA256ProcessBlock (pContext, pContext->buffer, size * 4);
+
+   // Get digest
+   int   i;
+   char *r = (char*)pDigest;
+
+   for (i = 0; i < 8; i++)
+      set_uint32 (r + i * sizeof pContext->state[0], SWAP (pContext->state[i]));
+}
+
+
+void SHA256Append (t_pSHA256Context pContext, const void *buffer, size_t len)
+{
+   // When we already have some bits in our internal buffer concatenate both inputs first
+   if (pContext->buflen != 0)
+   {
+      size_t left_over = pContext->buflen;
+      size_t add = 128 - left_over > len ? len : 128 - left_over;
+
+      memcpy (&((char *) pContext->buffer)[left_over], buffer, add);
+      pContext->buflen += add;
+
+      if (pContext->buflen > 64)
+      {
+         SHA256ProcessBlock (pContext, pContext->buffer, pContext->buflen & ~63);
+
+         pContext->buflen &= 63;
+         memcpy (pContext->buffer,
+                 &((char *) pContext->buffer)[(left_over + add) & ~63],
+                 pContext->buflen);
+      }
+      buffer = (const char *) buffer + add;
+      len -= add;
+   }
+
+   // Process available complete blocks
+   if (len >= 64)
+   {
+      #if !_STRING_ARCH_unaligned
+         #define alignof(type) offsetof (struct { char c; type x; }, x)
+         #define UNALIGNED_P(p) (((size_t) p) % alignof (uint32) != 0)
+         if (UNALIGNED_P (buffer))
+         {
+            while (len > 64)
+            {
+               SHA256ProcessBlock (pContext, memcpy (pContext->buffer, buffer, 64), 64);
+               buffer = (const char *) buffer + 64;
+               len -= 64;
+            }
+         }
+         else
+      #endif
+         {
+            SHA256ProcessBlock (pContext, buffer, len & ~63);
+            buffer = (const char *) buffer + (len & ~63);
+            len &= 63;
+         }
+   }
+
+   // Move remaining bytes in internal buffer
+   if (len > 0)
+   {
+      size_t left_over = pContext->buflen;
+
+      memcpy (&((char *) pContext->buffer)[left_over], buffer, len);
+      left_over += len;
+      if (left_over >= 64)
+      {
+         SHA256ProcessBlock (pContext, pContext->buffer, 64);
+         left_over -= 64;
+         memcpy (pContext->buffer, &pContext->buffer[16], left_over);
+      }
+      pContext->buflen = left_over;
+   }
+}
+
+
+#endif // SHA256_OLD
+
 #ifdef TEST
 
 #include <stdlib.h>
@@ -576,82 +880,64 @@ static const char *val[] =
 
 int main( int argc, char *argv[] )
 {
-    FILE *f;
-    int i, j;
-    char output[65];
-    sha256_context ctx;
-    unsigned char buf[65536];
-    unsigned char sha256sum[32];
+   FILE            *f;
+   int             i, j;
+   char            output[65];
+   t_SHA256Context Context;
+   unsigned char   buf[65536];
+   unsigned char   sha256sum[32];
 
-    if( argc < 2 )
-    {
-        printf( "\n SHA-256 Validation Tests:\n\n" );
+   if( argc < 2 )
+   {
+      printf( "\n SHA-256 Validation Tests:\n\n" );
 
-        for( i = 0; i < 3; i++ )
-        {
-            printf( " Test %d ", i + 1 );
+      for( i = 0; i < 3; i++ )
+      {
+         printf( " Test %d ", i + 1 );
+         SHA256Init (&Context);
+         if( i < 2 )
+         {
+            SHA256Append (&Context, (uint8 *) msg[i], strlen( msg[i] ) );
+         }
+         else
+         {
+            memset( buf, 'a', 1000 );
+            for (j = 0; j < 1000; j++)
+               SHA256Append (&Context, (uint8 *) buf, 1000 );
+         }
 
-            sha256_starts( &ctx );
+         SHA256Finish (&Context, sha256sum);
+         for (j = 0; j < 32; j++)
+            sprintf( output + j * 2, "%02x", sha256sum[j] );
 
-            if( i < 2 )
-            {
-                sha256_update( &ctx, (uint8 *) msg[i],
-                               strlen( msg[i] ) );
-            }
-            else
-            {
-                memset( buf, 'a', 1000 );
-
-                for( j = 0; j < 1000; j++ )
-                {
-                    sha256_update( &ctx, (uint8 *) buf, 1000 );
-                }
-            }
-
-            sha256_finish( &ctx, sha256sum );
-
-            for( j = 0; j < 32; j++ )
-            {
-                sprintf( output + j * 2, "%02x", sha256sum[j] );
-            }
-
-            if( memcmp( output, val[i], 64 ) )
-            {
-                printf( "failed!\n" );
-                return( 1 );
-            }
-
-            printf( "passed.\n" );
-        }
-
-        printf( "\n" );
-    }
-    else
-    {
-        if( ! ( f = fopen( argv[1], "rb" ) ) )
-        {
-            perror( "fopen" );
+         if (memcmp( output, val[i], 64 ))
+         {
+            printf( "failed!\n" );
             return( 1 );
-        }
+         }
+         printf( "passed.\n" );
+      }
+      printf( "\n" );
+   }
+   else
+   {
+      if (!(f = fopen( argv[1], "rb")))
+      {
+         perror( "fopen" );
+         return( 1 );
+     }
 
-        sha256_starts( &ctx );
+     SHA256Init (&Context);
+     while ((i = fread(buf, 1, sizeof(buf), f)) > 0)
+        SHA256Append (&Context, buf, i );
+     SHA256Finish (&Context, sha256sum);
 
-        while( ( i = fread( buf, 1, sizeof( buf ), f ) ) > 0 )
-        {
-            sha256_update( &ctx, buf, i );
-        }
+     for (j = 0; j<32; j++)
+        printf( "%02x", sha256sum[j] );
 
-        sha256_finish( &ctx, sha256sum );
-
-        for( j = 0; j < 32; j++ )
-        {
-            printf( "%02x", sha256sum[j] );
-        }
-
-        printf( "  %s\n", argv[1] );
-    }
-
-    return( 0 );
+     printf( "  %s\n", argv[1] );
+   }
+   return (0);
 }
 
 #endif
