@@ -9,7 +9,7 @@
 //  Module:         The table widget (central widget of the application)
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -135,8 +135,9 @@ APIRET t_Table::GetDeviceUnderCursor (t_pDevice &pDevice)
 void t_Table::contextMenuEvent (QContextMenuEvent *pEvent)
 {
    QAction     *pAction;
-   QAction       LocalIndicator (tr("Local Device - cannot be acquired"           ), this);
-   QAction       CloneIndicator (tr("Device used for cloning - cannot be acquired"), this);
+   QAction       LocalIndicator        (tr("Local Device - cannot be acquired"                   ), this);
+   QAction       CloneIndicator        (tr("Device used for cloning - cannot be acquired"        ), this);
+   QAction       AddStateInfoIndicator (tr("Cannot be acquired - see additional state info field"), this);
    QMenu         Menu;
    QPoint        Pos;
    t_pDevice    pDevice;
@@ -177,6 +178,12 @@ void t_Table::contextMenuEvent (QContextMenuEvent *pEvent)
       Menu.addAction (&CloneIndicator);
       Menu.addSeparator ();
    }
+   else if (!pDevice->AddStateInfo.CanBeAcquired)
+   {
+      AddStateInfoIndicator.setEnabled (false);
+      Menu.addAction (&AddStateInfoIndicator);
+      Menu.addSeparator ();
+   }
    else
    {
       Menu.addAction (pOwn->pActionAcquire);
@@ -219,7 +226,7 @@ APIRET t_Table::ShowDeviceInfo (t_pcDevice pDevice)
 {
    QString Info;
 
-   LOG_INFO ("Info for %s requested", QSTR_TO_PSZ(pDevice->LinuxDevice))
+   LOG_INFO ("[%s] Info for requested", QSTR_TO_PSZ(pDevice->LinuxDevice))
    pOwn->SlowDownAcquisitions = true;
    CHK (t_Info::GetDeviceInfo (pDevice, true, Info))
    pOwn->SlowDownAcquisitions = false;
@@ -314,19 +321,20 @@ APIRET TableFifoCalcSizeAndAlloc (t_pDevice pDevice, unsigned int Fifos, unsigne
 {
    const unsigned int PageSize = getpagesize();
    unsigned int       MaxBlocksInUse;
+   unsigned int       ExtraSpace;
 
    // Set the data block size
    // -----------------------
    switch (pDevice->Acquisition.Format)
    {
-      case t_File::DD : pDevice->FifoBlockSize = CONFIG(FifoBlockSizeDD );   break;
-      case t_File::AFF: pDevice->FifoBlockSize = CONFIG(FifoBlockSizeAFF);   break;
+      case t_File::DD  : pDevice->FifoBlockSize = CONFIG(FifoBlockSizeDD );   break;
+      case t_File::AAFF: pDevice->FifoBlockSize = CONFIG(FifoBlockSizeAFF);   break;
       case t_File::EWF:
-         if (pDevice->HasCompressionThreads())
+         if (pDevice->HasCompressionThreads() && (CONFIG(EwfFormat) != t_File::AEWF))
               pDevice->FifoBlockSize = EWF_MULTITHREADED_COMPRESSION_CHUNK_SIZE;
          else pDevice->FifoBlockSize = CONFIG(FifoBlockSizeEWF);
          if (pDevice->FifoBlockSize != (unsigned int) CONFIG(FifoBlockSizeEWF))
-            LOG_INFO ("Running with FifoBlockSize of %d (no other size supported when running with multithreaded EWF compression)", pDevice->FifoBlockSize)
+            LOG_INFO ("[%s] Running with FifoBlockSize of %d (no other size supported when using libewf with multithreaded compression)", QSTR_TO_PSZ (pDevice->LinuxDevice), pDevice->FifoBlockSize)
          break;
       default:  CHK (ERROR_TABLE_INVALID_FORMAT)
    }
@@ -335,8 +343,8 @@ APIRET TableFifoCalcSizeAndAlloc (t_pDevice pDevice, unsigned int Fifos, unsigne
    // ---------------------------------------------------
    pDevice->FifoAllocBlockSize = pDevice->FifoBlockSize;    // Start with the standard data size
 
-   if (pDevice->HasCompressionThreads())                    // Add what might be necessary for compressed  data
-      pDevice->FifoAllocBlockSize += (int)(pDevice->FifoBlockSize * 0.001) + 12;  // Reserve enough room so that compressed data its in the block.
+   CHK (t_ThreadCompress::GetFifoExtraSpace (pDevice, &ExtraSpace))
+   pDevice->FifoAllocBlockSize += ExtraSpace;
 
    pDevice->FifoAllocBlockSize += sizeof (t_FifoBlock);     // Add the structure overhead
 
@@ -356,7 +364,7 @@ APIRET TableFifoCalcSizeAndAlloc (t_pDevice pDevice, unsigned int Fifos, unsigne
    // --------------------------------------------------------
    MaxBlocksInUse = pDevice->FifoMaxBlocks * Fifos;  // Data in the FIFOs
    MaxBlocksInUse += 1;                              // ThreadRead might allocate an additional block that can't yet be written to the full FIFO
-   if ((pDevice->Acquisition.Format == t_File::AFF) && !pDevice->HasCompressionThreads())
+   if ((pDevice->Acquisition.Format == t_File::AAFF) && !pDevice->HasCompressionThreads())
       MaxBlocksInUse += 1;                           // ThreadWrite might use an additional buffer if it does the AFF preprocessing
    if (pDevice->HasCompressionThreads())
       MaxBlocksInUse += CONFIG(CompressionThreads);  // Each compression thread might have one additional block for writing the compressed data
@@ -379,13 +387,13 @@ APIRET t_Table::StartAcquisition (t_pDevice pDevice, bool Clone)
       return NO_ERROR;
 
    pOwn->SlowDownAcquisitions = true;
-   LOG_INFO ("Starting acquisition (%s) for %s", Clone ? "clone":"image", QSTR_TO_PSZ(pDevice->LinuxDevice))
+   LOG_INFO ("[%s] Starting acquisition (%s)", QSTR_TO_PSZ (pDevice->LinuxDevice), Clone ? "clone":"image")
    CHK (Dlg.GetParameters (pDevice->Acquisition))
-   CHK (t_File::GetFormatExtension  (pDevice->Acquisition.Format, pDevice->Acquisition.Clone, NULL, &Format))
+   CHK (t_File::GetFormatExtension  (pDevice->Acquisition.Format, pDevice->Acquisition.Clone, 0, NULL, &Format))
    if (Clone)
       Format = " (clone)";
-   LOG_INFO ("Image %s%s%s", QSTR_TO_PSZ(pDevice->Acquisition.ImagePath), QSTR_TO_PSZ(pDevice->Acquisition.ImageFilename), QSTR_TO_PSZ(Format))
-   LOG_INFO ("Info  %s%s%s", QSTR_TO_PSZ(pDevice->Acquisition.InfoPath ), QSTR_TO_PSZ(pDevice->Acquisition.InfoFilename ), t_File::pExtensionInfo)
+   LOG_INFO ("[%s] Image %s%s%s", QSTR_TO_PSZ (pDevice->LinuxDevice), QSTR_TO_PSZ(pDevice->Acquisition.ImagePath), QSTR_TO_PSZ(pDevice->Acquisition.ImageFilename), QSTR_TO_PSZ(Format))
+   LOG_INFO ("[%s] Info  %s%s%s", QSTR_TO_PSZ (pDevice->LinuxDevice), QSTR_TO_PSZ(pDevice->Acquisition.InfoPath ), QSTR_TO_PSZ(pDevice->Acquisition.InfoFilename ), t_File::pExtensionInfo)
 
    CHK (pDevice->SetMessage (QString()))
    pDevice->State               = t_Device::Acquire;
@@ -409,6 +417,7 @@ APIRET t_Table::StartAcquisition (t_pDevice pDevice, bool Clone)
    memset (&pDevice->SHA256Digest         , 0, sizeof(pDevice->SHA256Digest         ));
    memset (&pDevice->SHA256DigestVerifySrc, 0, sizeof(pDevice->SHA256DigestVerifySrc));
    memset (&pDevice->SHA256DigestVerifyDst, 0, sizeof(pDevice->SHA256DigestVerifyDst));
+   pDevice->ImageFileHashList.clear();
    pDevice->StartTimestamp = QDateTime::currentDateTime();
    CHK (InfoAcquisitionStart (pDevice))
 
@@ -490,10 +499,12 @@ APIRET t_Table::StartAcquisition (t_pDevice pDevice, bool Clone)
       pDevice->pFifoWrite       = pDevice->pFifoCompressOut;
    }
 
-   LOG_INFO ("Acquisition runs with %d fifos, each one with space for %d blocks of %d bytes (%0.1f MB FIFO space)",
+   LOG_INFO ("[%s] Acquisition runs with %d fifos, each one with space for %d blocks of %d bytes (%0.1f MB FIFO space)",
+             QSTR_TO_PSZ (pDevice->LinuxDevice),
              Fifos, pDevice->FifoMaxBlocks, pDevice->FifoBlockSize,
              ((double)Fifos * pDevice->FifoMaxBlocks * pDevice->FifoBlockSize) / BYTES_PER_MEGABYTE)
-   LOG_INFO ("Including the overhead, %u blocks of %d bytes have been allocated (%0.1f MB)",
+   LOG_INFO ("[%s] Including the overhead, %u blocks of %d bytes have been allocated (%0.1f MB)",
+             QSTR_TO_PSZ (pDevice->LinuxDevice),
              MaxBlocksInUse, pDevice->FifoAllocBlockSize,
              ((double)MaxBlocksInUse * pDevice->FifoAllocBlockSize) / BYTES_PER_MEGABYTE)
 
@@ -532,8 +543,8 @@ APIRET t_Table::InfoAcquisitionBadSectors (t_pDevice pDevice, bool Verify)
    if (Count)
    {
       if (Verify)
-           LOG_INFO ("During verification, %lld bad sectors have been encountered", Count)
-      else LOG_INFO ("During acquisition, %lld bad sectors have been encountered", Count)
+           LOG_INFO ("[%s] During verification, %lld bad sectors have been encountered", QSTR_TO_PSZ (pDevice->LinuxDevice), Count)
+      else LOG_INFO ("[%s] During acquisition, %lld bad sectors have been encountered", QSTR_TO_PSZ (pDevice->LinuxDevice), Count)
 
       if (Verify)
            CHK (pDevice->Info.WriteLn (tr("During verification, %1 bad sectors have been encountered. The sector numbers are:", "Info file") .arg(Count)))
@@ -582,12 +593,12 @@ APIRET t_Table::InfoAcquisitionBadSectors (t_pDevice pDevice, bool Verify)
    {
       if (Verify)
       {
-         LOG_INFO ("No bad sectors encountered during verification.")
+         LOG_INFO ("[%s] No bad sectors encountered during verification.", QSTR_TO_PSZ (pDevice->LinuxDevice))
          CHK (pDevice->Info.WriteLn (tr("No bad sectors encountered during verification.", "Info file")))
       }
       else
       {
-         LOG_INFO ("No bad sectors encountered during acquisition.")
+         LOG_INFO ("[%s] No bad sectors encountered during acquisition.", QSTR_TO_PSZ (pDevice->LinuxDevice))
          CHK (pDevice->Info.WriteLn (tr("No bad sectors encountered during acquisition.", "Info file")))
       }
    }
@@ -604,19 +615,20 @@ static APIRET TableTimestampToISO (const QDateTime &Timestamp, QString &Str)
 
 APIRET t_Table::InfoAcquisitionEnd (t_pDevice pDevice)
 {
-   QString    MD5;
-   QString    MD5VerifySrc;
-   QString    MD5VerifyDst;
-   QString    SHA256;
-   QString    SHA256VerifySrc;
-   QString    SHA256VerifyDst;
-   QString    StateStr;
-   quint64    BadSectors;
-   QString    StartStr, StopStr, VerifyStr;
-   int        Hours, Minutes, Seconds;
-   double     Speed;
-   bool       MatchSrc = true;
-   bool       MatchDst = true;
+   QString MD5;
+   QString MD5VerifySrc;
+   QString MD5VerifyDst;
+   QString SHA256;
+   QString SHA256VerifySrc;
+   QString SHA256VerifyDst;
+   QString StateStr;
+   quint64 BadSectors;
+   QString StartStr, StopStr, VerifyStr;
+   QString Filename;
+   int     Hours, Minutes, Seconds;
+   double  Speed;
+   bool    MatchSrc = true;
+   bool    MatchDst = true;
 
    CHK (pDevice->Info.WriteLn ())
    CHK (InfoAcquisitionBadSectors (pDevice, false))
@@ -666,8 +678,12 @@ APIRET t_Table::InfoAcquisitionEnd (t_pDevice pDevice)
                   if (pDevice->Acquisition.CalcSHA256) CHK (HashSHA256DigestStr (&pDevice->SHA256Digest, SHA256)) // acquisition), we can calculate the acquisition hash.
                }
                break;
+            case t_Device::ThreadWriteVerifyError:
+               StateStr = tr("Aborted because of image read error during verification", "Info file");
+               if (pDevice->Acquisition.CalcMD5   ) CHK (HashMD5DigestStr    (&pDevice->MD5Digest   , MD5   ))
+               if (pDevice->Acquisition.CalcSHA256) CHK (HashSHA256DigestStr (&pDevice->SHA256Digest, SHA256))
+               break;
             case t_Device::ThreadWriteWriteError : StateStr = tr("Aborted because of image write error"                   , "Info file"); break;
-            case t_Device::ThreadWriteVerifyError: StateStr = tr("Aborted because of image read error during verification", "Info file"); break;
             default:                               StateStr = tr("Aborted, strange reason (%1)"                           , "Info file") .arg(pDevice->AbortReason); break;
          }
          break;
@@ -757,6 +773,44 @@ APIRET t_Table::InfoAcquisitionEnd (t_pDevice pDevice)
       CHK (pDevice->Info.AddRow (tr("Acquisition speed:: %1 MByte/s (%2 hours, %3 minutes and %4 seconds)", "Info file").arg(Speed, 0, 'f', 2) .arg(Hours) .arg(Minutes) .arg(Seconds)))
       CHK (pDevice->Info.WriteTable ());
    }
+
+   // Images files and hashes
+   // -----------------------
+   t_Device::t_ImageFileHash *pFileHash;
+
+   CHK (pDevice->Info.WriteLn ())
+   CHK (pDevice->Info.WriteLn ())
+   CHK (pDevice->Info.Title   (tr("Generated image files and their MD5 hashes", "Info file")))
+   CHK (pDevice->Info.WriteLn ())
+   if (CONFIG(CalcImageFileMD5))
+   {
+      if (pDevice->Acquisition.VerifyDst)
+      {
+         if (pDevice->Acquisition.Format == t_File::EWF)
+            CHK (pDevice->Info.WriteLn (tr("No MD5 hashes available (the select EWF format does not support image file hashes)")))
+      }
+      else
+      {
+         CHK (pDevice->Info.WriteLn (tr("No MD5 hashes available (image verification checkbox was switched off)")))
+      }
+   }
+   else
+   {
+      CHK (pDevice->Info.WriteLn (tr("No MD5 hashes available (configuration parameter CalcImageFileMD5 is off)")))
+   }
+   CHK (pDevice->Info.WriteLn ("MD5                               Image file"))
+   for (int i=0; i<pDevice->ImageFileHashList.count(); i++)
+   {
+      pFileHash = pDevice->ImageFileHashList[i];
+      if (pFileHash->MD5Valid)
+           CHK (HashMD5DigestStr(&pFileHash->MD5Digest, MD5))
+      else MD5 = "n/a                             ";
+      Filename = QFileInfo(pFileHash->Filename).fileName();
+      CHK (pDevice->Info.WriteLn (QString("%1  %2") .arg(MD5) .arg(Filename)))
+   }
+
+
+
    CHK (pDevice->Info.WriteLn ());
    CHK (pDevice->Info.WriteLn ());
 
@@ -807,7 +861,6 @@ APIRET t_Table::FinaliseThreadStructs (t_pDevice pDevice)
 //      LibEwfGetMemStats (&LibEwfAllocs, &LibEwfFrees);
 //      LOG_INFO ("LIBEWF mem  statistics: %d allocated - %d freed = %d remaining", LibEwfAllocs, LibEwfFrees, LibEwfAllocs - LibEwfFrees)
 
-      LOG_INFO ("Acquisition of %s: All structures cleaned.", QSTR_TO_PSZ (pDevice->LinuxDevice))
 
       if (pDevice->AbortRequest)
            pDevice->State = t_Device::Aborted;
@@ -816,6 +869,11 @@ APIRET t_Table::FinaliseThreadStructs (t_pDevice pDevice)
       if (!pDevice->DeleteAfterAbort)
          CHK_EXIT (InfoAcquisitionEnd (pDevice))
 
+      while (!pDevice->ImageFileHashList.isEmpty())
+         delete pDevice->ImageFileHashList.takeFirst();
+
+      LOG_INFO ("[%s] All structures cleaned.", QSTR_TO_PSZ (pDevice->LinuxDevice))
+
       // Check if all acqusitions have completed now (for AutoExit)
       // ----------------------------------------------------------
       t_Device::t_State State;
@@ -838,7 +896,7 @@ APIRET t_Table::FinaliseThreadStructs (t_pDevice pDevice)
 
 void t_Table::SlotThreadReadFinished (t_pDevice pDevice)
 {
-   LOG_INFO ("Acquisition of %s: Read thread finished", QSTR_TO_PSZ (pDevice->LinuxDevice))
+   LOG_INFO ("[%s] Read thread finished", QSTR_TO_PSZ (pDevice->LinuxDevice))
    pDevice->pThreadRead->deleteLater();
    pDevice->pThreadRead = NULL;
 
@@ -850,7 +908,7 @@ void t_Table::SlotThreadReadFinished (t_pDevice pDevice)
 
 void t_Table::SlotThreadHashFinished (t_pDevice pDevice)
 {
-   LOG_INFO ("Acquisition of %s: Hash thread finished", QSTR_TO_PSZ (pDevice->LinuxDevice))
+   LOG_INFO ("[%s] Hash thread finished", QSTR_TO_PSZ (pDevice->LinuxDevice))
    pDevice->pThreadHash->deleteLater();
    pDevice->pThreadHash = NULL;
 
@@ -864,7 +922,7 @@ void t_Table::SlotThreadCompressFinished (t_pDevice pDevice, int ThreadNr)
 {
    bool NoMoreThreads = true;
 
-   LOG_INFO ("Acquisition of %s: Compression thread #%d finished", QSTR_TO_PSZ (pDevice->LinuxDevice), ThreadNr)
+   LOG_INFO ("[%s] Compression thread #%d finished", QSTR_TO_PSZ (pDevice->LinuxDevice), ThreadNr)
 
    pDevice->ThreadCompressList[ThreadNr]->deleteLater();
    pDevice->ThreadCompressList[ThreadNr] = NULL;
@@ -876,7 +934,7 @@ void t_Table::SlotThreadCompressFinished (t_pDevice pDevice, int ThreadNr)
 
    if (NoMoreThreads)
    {
-       LOG_INFO ("All %d compression threads finished", pDevice->ThreadCompressList.count())
+       LOG_INFO ("[%s] All %d compression threads finished", QSTR_TO_PSZ (pDevice->LinuxDevice), pDevice->ThreadCompressList.count())
        pDevice->ThreadCompressList.clear();
    }
 
@@ -889,7 +947,7 @@ void t_Table::SlotThreadCompressFinished (t_pDevice pDevice, int ThreadNr)
 
 void t_Table::SlotThreadWriteFinished (t_pDevice pDevice)
 {
-   LOG_INFO ("Acquisition of %s: Write thread finished", QSTR_TO_PSZ (pDevice->LinuxDevice))
+   LOG_INFO ("[%s] Write thread finished", QSTR_TO_PSZ (pDevice->LinuxDevice))
 
    pDevice->pThreadWrite->deleteLater();
    pDevice->pThreadWrite = NULL;
@@ -902,11 +960,11 @@ void t_Table::SlotThreadWriteFinished (t_pDevice pDevice)
 
 void t_Table::SlotThreadWriteWakeThreads (t_pDevice pDevice)
 {
-   LOG_INFO ("Acquisition of %s: Write thread asks other threads for releasing its handle", QSTR_TO_PSZ (pDevice->LinuxDevice))
+   LOG_INFO ("[%s] Write thread asks other threads for releasing its handle", QSTR_TO_PSZ (pDevice->LinuxDevice))
 
    if (pDevice->AbortRequest)
    {
-      LOG_INFO ("Acquisition of %s: Calling WakeWaitingThreads", QSTR_TO_PSZ (pDevice->LinuxDevice))
+      LOG_INFO ("[%s] Calling WakeWaitingThreads", QSTR_TO_PSZ (pDevice->LinuxDevice))
       CHK_EXIT (WakeWaitingThreads(pDevice))
    }
 }
@@ -915,11 +973,11 @@ APIRET t_Table::AbortAcquisition0 (t_pDevice pDevice)
 {
    if (pDevice->AbortRequest)
    {
-      LOG_INFO ("User pressed abort, but abort flag for acquisition on %s is already set.", QSTR_TO_PSZ(pDevice->LinuxDevice))
+      LOG_INFO ("[%s] User pressed abort, but abort flag is already set.", QSTR_TO_PSZ(pDevice->LinuxDevice))
       return NO_ERROR;
    }
 
-   LOG_INFO ("User aborts acquisition on %s", QSTR_TO_PSZ(pDevice->LinuxDevice))
+   LOG_INFO ("[%s] User aborts acquisition", QSTR_TO_PSZ(pDevice->LinuxDevice))
    pDevice->AbortReason  = t_Device::UserRequest;
    pDevice->AbortRequest = true;
 
@@ -989,4 +1047,3 @@ APIRET t_Table::WakeWaitingThreads (t_pDevice pDevice)
 
    return NO_ERROR;
 }
-
