@@ -9,7 +9,7 @@
 //  Module:         Multi-threaded compression
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -29,7 +29,7 @@
 #ifndef __THREADCOMPRESS_H__
 #define __THREADCOMPRESS_H__
 
-#include <QThread>   //lint !e537 Repeated include
+#include "thread.h"
 
 #ifndef __DEVICE_H__
   #include "device.h"
@@ -37,7 +37,7 @@
 
 class t_ThreadCompressLocal;
 
-class t_ThreadCompress: public QThread
+class t_ThreadCompress: public t_Thread
 {
    Q_OBJECT
 
@@ -46,6 +46,8 @@ class t_ThreadCompress: public QThread
       t_ThreadCompress (t_pDevice pDevice, int ThreadNr);
      ~t_ThreadCompress ();
 
+      static APIRET GetFifoExtraSpace (t_pDevice pDevice, unsigned int *pExtraSpace);
+
    protected:
       void run (void);
 
