@@ -9,7 +9,7 @@
 //  Module:         Thread for calculating hashes
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -80,7 +80,8 @@ void t_ThreadHash::run (void)
    t_HashContextSHA256 HashContextSHA256;
    bool                VerifyLoop = false;
 
-   LOG_INFO ("Acquisition of %s: Hash thread started", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice))
+   LOG_INFO ("[%s] Hash thread started", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice))
+   CHK_EXIT (SetDebugMessage ("Start run function"))
    pDevice = pOwn->pDevice;
    pBlocks = &BlocksCalculated;
    for (;;)  // The whole loop is done 2 times if the user chose to do a source verification, 1 time otherwise.
@@ -91,7 +92,9 @@ void t_ThreadHash::run (void)
       if (pDevice->Acquisition.CalcSHA256) CHK_EXIT (HashSHA256Init (&HashContextSHA256))
       do
       {
+         CHK_EXIT (SetDebugMessage ("Wait for block"))
          CHK_EXIT (pDevice->pFifoHashIn->Get (pFifoBlock))
+         CHK_EXIT (SetDebugMessage ("Process block"))
          if (pFifoBlock)
          {
             (*pBlocks)++;
@@ -109,7 +112,7 @@ void t_ThreadHash::run (void)
          }
          else
          {
-            LOG_INFO ("Dummy block")
+            LOG_INFO ("[%s] Dummy block", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice))
             Finished = true;
          }
       } while (!Finished && !pDevice->AbortRequest);
@@ -141,8 +144,9 @@ void t_ThreadHash::run (void)
    }
 
    if (pDevice->Acquisition.VerifySrc)
-        LOG_INFO ("Hash thread exits now (device %s, %Ld blocks processed, %Ld blocks verified)", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice), BlocksCalculated, BlocksVerified)
-   else LOG_INFO ("Hash thread exits now (device %s, %Ld blocks processed)"                     , QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice), BlocksCalculated)
+        LOG_INFO ("[%s] Hash thread exits now - %Ld blocks processed, %Ld blocks verified", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice), BlocksCalculated, BlocksVerified)
+   else LOG_INFO ("[%s] Hash thread exits now - %Ld blocks processed"                     , QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice), BlocksCalculated)
+   CHK_EXIT (SetDebugMessage ("Exit run function"))
 }
 
 
