@@ -9,7 +9,7 @@
 //  Module:         Thread for reading data.
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -113,7 +113,7 @@ static APIRET ThreadReadDeviceDisconnected (t_pDevice pDevice, const char* pStrA
    if (pDevice->State == t_Device::Acquire)
         pDevice->State = t_Device::AcquirePaused;
    else pDevice->State = t_Device::VerifyPaused;
-   LOG_INFO ("Device %s disconnected (%s, %d, %s), switching device state to %s", QSTR_TO_PSZ(pDevice->LinuxDevice), pStrAction, errno, strerror(errno), pDevice->StateStr())
+   LOG_INFO ("[%s] Device disconnected (%s, %d, %s), switching device state to %s", QSTR_TO_PSZ(pDevice->LinuxDevice), pStrAction, errno, strerror(errno), pDevice->StateStr())
 
    return ERROR_THREADREAD_DEVICE_DISCONNECTED;
 }
@@ -146,9 +146,9 @@ static APIRET ThreadReadBlock0 (t_pDevice pDevice, unsigned char *pBuffer, unsig
    if (pDevice->FileDescSrc == t_Device::FileDescEmpty)
    {
       OpenFlags = O_RDONLY | O_NOATIME;
-      if (CONFIG (FifoMemoryManager))     // We need the buffers to be located on page size (4K) boundaries; this only
-      {                                   // is guaranteed when working with Guymager's internal memory manager (see t_FifoMemory::t_FifoMemory)
-         LOG_INFO ("Trying direct mode for %s", QSTR_TO_PSZ(pDevice->LinuxDevice))
+      if (CONFIG (FifoMemoryManager) && CONFIG (DirectIO))  // We need the buffers to be located on page size (4K) boundaries; this only
+      {                                                     // is guaranteed when working with Guymager's internal memory manager (see t_FifoMemory::t_FifoMemory)
+         LOG_INFO ("[%s] Trying direct mode", QSTR_TO_PSZ(pDevice->LinuxDevice))
          OpenFlags |= O_DIRECT;
       }
       pDevice->FileDescSrc = open64 (QSTR_TO_PSZ(pDevice->LinuxDevice), OpenFlags);
@@ -157,7 +157,7 @@ static APIRET ThreadReadBlock0 (t_pDevice pDevice, unsigned char *pBuffer, unsig
          OpenFlags &= ~O_DIRECT;                                                       // (added as special device) on the cases server (dd reported
          pDevice->FileDescSrc = open64 (QSTR_TO_PSZ(pDevice->LinuxDevice), OpenFlags); // the same error with iflag=direct). Try again without it.
       }
-      LOG_INFO ("Direct mode for %s switched %s", QSTR_TO_PSZ(pDevice->LinuxDevice), (OpenFlags & O_DIRECT) ? "on":"off")
+      LOG_INFO ("[%s] Direct mode switched %s", QSTR_TO_PSZ(pDevice->LinuxDevice), (OpenFlags & O_DIRECT) ? "on":"off")
       if (pDevice->FileDescSrc < 0)
          CHK_RET (ThreadReadDeviceDisconnected (pDevice, "open"))
       CHK_RET (ThreadReadAdjustSeek (pDevice))
@@ -241,7 +241,7 @@ APIRET t_ThreadRead::ThreadReadBlock (t_pFifoBlock &pFifoBlock)
                pDevice->FallbackMode = true;
 //               ReadTry = (int) GETMIN (pDevice->SectorSize, pDevice->SectorSizePhys);
                ReadTry = pDevice->SectorSize;
-               LOG_INFO ("Device read error, switching %s to slow fallback mode. Reading sectors individually from now on.", QSTR_TO_PSZ(pDevice->LinuxDevice))
+               LOG_INFO ("[%s] Device read error, switching to slow fallback mode. Reading sectors individually from now on.", QSTR_TO_PSZ(pDevice->LinuxDevice))
             }
             else
             {
@@ -251,12 +251,12 @@ APIRET t_ThreadRead::ThreadReadBlock (t_pFifoBlock &pFifoBlock)
                switch (RcSeek)                          // the source device, let's replace it by a zero sector.
                {
                   case NO_ERROR:
-                     if ((pDevice->Acquisition.Format == t_File::AFF) && (CONFIG (AffMarkBadSectors)))
+                     if ((pDevice->Acquisition.Format == t_File::AAFF) && (CONFIG (AffMarkBadSectors)))
                           CHK (AaffCopyBadSectorMarker (pBuffer, ReadTry))
                      else memset (pBuffer, 0, ReadTry);
                      BytesRead += ReadTry;
                      pBuffer   += ReadTry;
-                     LOG_INFO ("Read error on %s, sector %Ld, bad data replaced by zero block", QSTR_TO_PSZ(pDevice->LinuxDevice), Sector)
+                     LOG_INFO ("[%s] Read error on sector %Ld, bad data replaced by zero block", QSTR_TO_PSZ(pDevice->LinuxDevice), Sector)
                      pDevice->AddBadSector (Sector);
                      break;
                   case ERROR_THREADREAD_DEVICE_DISCONNECTED:
@@ -294,7 +294,7 @@ APIRET t_ThreadRead::ThreadReadBlock (t_pFifoBlock &pFifoBlock)
    else if ((pDevice->FallbackMode) && (RcRead == NO_ERROR))
    {
       pDevice->FallbackMode = false;  // Switch to fast reading mode if last read was good
-      LOG_INFO ("Device read ok again. Switching %s to fast block read.", QSTR_TO_PSZ(pDevice->LinuxDevice))
+      LOG_INFO ("[%s] Device read ok again, switching to fast block read.", QSTR_TO_PSZ(pDevice->LinuxDevice))
    }
 
    pFifoBlock->LastBlock = ThreadReadEOF (pDevice);
@@ -318,12 +318,19 @@ void t_ThreadRead::run (void)
    t_HashContextMD5    HashContextMD5;
    t_HashContextSHA256 HashContextSHA256;
 
-   LOG_INFO ("Acquisition of %s: Read thread started", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice))
+   LOG_INFO ("[%s] Read thread started", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice))
+   CHK_EXIT (SetDebugMessage ("Start run function"))
    pDevice = pOwn->pDevice;
    CalcHashMD5    = pDevice->Acquisition.CalcMD5    && !CONFIG (UseSeparateHashThread);
    CalcHashSHA256 = pDevice->Acquisition.CalcSHA256 && !CONFIG (UseSeparateHashThread);
 
    pBlocks = &BlocksRead;
+
+//   pDevice->AddBadSector (3);   // Simulate bad sectors
+//   pDevice->AddBadSector (4);
+//   pDevice->AddBadSector (5);
+//   pDevice->AddBadSector (122);
+
    pDevice->State = t_Device::Acquire;
    for (;;)
    {
@@ -338,7 +345,9 @@ void t_ThreadRead::run (void)
          if (*(pOwn->pSlowDownRequest))
             msleep (THREADREAD_SLOWDOWN_SLEEP);
 
+         CHK_EXIT (SetDebugMessage ("Calling ThreadReadBlock"))
          rc = ThreadReadBlock (pFifoBlock);
+         CHK_EXIT (SetDebugMessage ("ThreadReadBlock returned"))
          switch (rc)
          {
             case NO_ERROR:
@@ -368,7 +377,7 @@ void t_ThreadRead::run (void)
             case ERROR_THREADREAD_DEVICE_ABORTED:
                break;
             default:
-               LOG_ERROR ("Unexpected return code ThreadReadBlock")
+               LOG_ERROR ("[%s] Unexpected return code from ThreadReadBlock", QSTR_TO_PSZ(pDevice->LinuxDevice))
                CHK_EXIT (rc)
          }
       } while (!ThreadReadEOF(pDevice) && !pDevice->AbortRequest);
@@ -384,7 +393,7 @@ void t_ThreadRead::run (void)
       {
          if (CalcHashMD5)    CHK_EXIT (HashMD5Digest    (&HashContextMD5   , &pDevice->MD5DigestVerifySrc   ))
          if (CalcHashSHA256) CHK_EXIT (HashSHA256Digest (&HashContextSHA256, &pDevice->SHA256DigestVerifySrc))
-         LOG_INFO ("Source verification completed (device %s)", QSTR_TO_PSZ (pDevice->LinuxDevice))
+         LOG_INFO ("[%s] Source verification completed", QSTR_TO_PSZ (pDevice->LinuxDevice))
          break;
       }
 
@@ -401,14 +410,14 @@ void t_ThreadRead::run (void)
       if (pDevice->Acquisition.VerifySrc)
       {
          (void) close (pDevice->FileDescSrc);
-         LOG_INFO ("All data has been read (device %s), now re-reading for source verification", QSTR_TO_PSZ (pDevice->LinuxDevice))
+         LOG_INFO ("[%s] All data has been read, now re-reading for source verification", QSTR_TO_PSZ (pDevice->LinuxDevice))
          pBlocks = &BlocksVerified;
 //         if (CONFIG (UseSeparateHashThread))
          CHK_EXIT (pDevice->pFifoRead->InsertDummy ())
       }
       else
       {
-         LOG_INFO ("All data has been read (device %s)", QSTR_TO_PSZ (pDevice->LinuxDevice))
+         LOG_INFO ("[%s] All data has been read", QSTR_TO_PSZ (pDevice->LinuxDevice))
          break;
       }
    }
@@ -419,15 +428,16 @@ void t_ThreadRead::run (void)
       pDevice->FileDescSrc = t_Device::FileDescEmpty;
       if (rcf)
       {
-         LOG_INFO ("Unexpected error on close, errno = %d", errno)
+         LOG_INFO ("[%s] Unexpected error on close, errno = %d", QSTR_TO_PSZ(pDevice->LinuxDevice), errno)
          pDevice->AbortReason  = t_Device::ThreadReadFileError;
          pDevice->AbortRequest = true;
       }
    }
 
-   LOG_INFO ("Read thread exits now (device %s, %Ld blocks read, %llu bytes)", QSTR_TO_PSZ (pDevice->LinuxDevice), BlocksRead, BytesRead)
+   LOG_INFO ("[%s] Read thread exits now (%Ld blocks read, %llu bytes)", QSTR_TO_PSZ (pDevice->LinuxDevice), BlocksRead, BytesRead)
    if (pDevice->Acquisition.VerifySrc)
-   LOG_INFO ("             (for the source verification: %Ld blocks read, %llu bytes)", BlocksVerified, BytesVerified)
+   LOG_INFO ("[%s]   Source verification: %Ld blocks read, %llu bytes)", QSTR_TO_PSZ(pDevice->LinuxDevice), BlocksVerified, BytesVerified)
+   CHK_EXIT (SetDebugMessage ("Exit run function"))
 }
 
 void t_ThreadRead::SlotFinished (void)
