@@ -9,7 +9,7 @@
 //  Module:         Thread for reading data.
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -29,7 +29,7 @@
 #ifndef __THREADREAD_H__
 #define __THREADREAD_H__
 
-#include <QThread>
+#include "thread.h"
 
 #ifndef __DEVICE_H__
    #include "device.h"
@@ -37,7 +37,7 @@
 
 class t_ThreadReadLocal;
 
-class t_ThreadRead: public QThread
+class t_ThreadRead: public t_Thread
 {
    Q_OBJECT
 
