@@ -9,7 +9,7 @@
 //  Module:         Thread for scanning the connected devices
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -70,20 +70,14 @@ t_ThreadScanWorker::t_ThreadScanWorker (t_ThreadScan *pThreadScan)
 //  Utility functions used by workers
 // -----------------------------------
 
-static APIRET ThreadScanMarkLocalDevices (const t_pDeviceList pDeviceList)
+static APIRET ThreadScanSetLocal (t_pDevice pDevice)
 {
    QStringList *pLocalDevices;
-   t_pDevice    pDevice;
-   int           i;
 
    CHK_EXIT (CfgGetLocalDevices (&pLocalDevices))
-   for (i=0; i<pDeviceList->count(); i++)
-   {
-      pDevice = pDeviceList->at (i);
-      pDevice->Local = pLocalDevices->contains (pDevice->SerialNumber) ||
-                       pLocalDevices->contains (pDevice->LinuxDevice ) ||
-                       pLocalDevices->contains (pDevice->Model       );
-   }
+   pDevice->Local = pLocalDevices->contains (pDevice->SerialNumber) ||
+                    pLocalDevices->contains (pDevice->LinuxDevice ) ||
+                    pLocalDevices->contains (pDevice->Model       );
    return NO_ERROR;
 }
 
@@ -97,8 +91,8 @@ static APIRET ThreadScanGetAddStateInfo (t_pDevice pDevice)
    Command = CONFIG (CommandGetAddStateInfo);
    if (!Command.isEmpty())
    {
-      Command.replace ("%dev", pDevice->LinuxDevice, Qt::CaseInsensitive);
-   
+      Command.replace ("%dev"  , pDevice->LinuxDevice    , Qt::CaseInsensitive);
+      Command.replace ("%local", pDevice->Local ? "1":"0", Qt::CaseInsensitive);
       rc = QtUtilProcessCommand (Command, StateInfo);
       if (rc == ERROR_QTUTIL_COMMAND_TIMEOUT)
       {
@@ -303,10 +297,10 @@ void t_ThreadScanWorkerParted::SlotRescan (void)
       pDevice = pDeviceList->AppendNew (SerialNumber, pPedDev);
       if (pDevice->LinuxDevice.startsWith ("/dev/fd",  Qt::CaseSensitive)) // Not a good way for checking this, but I don't know how to extract the information from
          pDevice->Removable = true;                                        // PedDevice. BTW, this won't work with other removable devices, for instance memory sticks.
+      CHK_EXIT (ThreadScanSetLocal        (pDevice));
       CHK_EXIT (ThreadScanGetAddStateInfo (pDevice));
    }
    pOwn->pFnLibPartedFreeAll ();
-   CHK_EXIT (ThreadScanMarkLocalDevices (pDeviceList))
 
    emit (SignalScanFinished (pDeviceList));
    CHK_EXIT (poThreadScan->SetDebugMessage ("Leaving SlotRescan (parted)"))
@@ -553,11 +547,12 @@ void t_ThreadScanWorkerHAL::SlotRescan (void)
                                            DEVICE_DEFAULT_SECTOR_SIZE, DEVICE_DEFAULT_SECTOR_SIZE, Size.toULongLong());
          pDevice->Removable = Removable.toBool();
 
-         CHK_EXIT (pDevice->MediaInfo.QueryDevice(QSTR_TO_PSZ(LinuxDevice.toString())))
+         if (CONFIG(QueryDeviceMediaInfo))
+            CHK_EXIT (pDevice->MediaInfo.QueryDevice(QSTR_TO_PSZ(LinuxDevice.toString())))
+         CHK_EXIT (ThreadScanSetLocal        (pDevice));
          CHK_EXIT (ThreadScanGetAddStateInfo (pDevice));
       }
    }
-   CHK_EXIT (ThreadScanMarkLocalDevices (pDeviceList))
    LOG_INFO ("Device scan finished")
 
    CHK_EXIT (poThreadScan->SetDebugMessage ("Leaving SlotRescan (HAL)"))
@@ -816,11 +811,12 @@ void t_ThreadScanWorkerDevKit::SlotRescan (void)
          pDevice = pDeviceList->AppendNew (SerialNumber.toString(), LinuxDevice.toString(), Model,
                                            BlockSize.toULongLong(), BlockSize.toULongLong(), Size.toULongLong());
          pDevice->Removable = Removable.toBool();
-         CHK_EXIT (pDevice->MediaInfo.QueryDevice(QSTR_TO_PSZ(LinuxDevice.toString())))
+         if (CONFIG(QueryDeviceMediaInfo))
+            CHK_EXIT (pDevice->MediaInfo.QueryDevice(QSTR_TO_PSZ(LinuxDevice.toString())))
+         CHK_EXIT (ThreadScanSetLocal        (pDevice));
          CHK_EXIT (ThreadScanGetAddStateInfo (pDevice));
       }
    }
-   CHK_EXIT (ThreadScanMarkLocalDevices (pDeviceList))
    LOG_INFO ("Device scan finished")
 
    CHK_EXIT (poThreadScan->SetDebugMessage ("Leaving SlotRescan (DevKit)"))
