@@ -7,7 +7,7 @@
 //                  Section Nouvelles Technologies
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -25,7 +25,8 @@
 // along with guymager. If not, see <http://www.gnu.org/licenses/>.
 
 #include <QObject>  //lint !e537 Repeated include
-#include <QThread>
+
+#include "thread.h"
 
 #ifndef __DEVICE_H__
    #include "device.h"
@@ -35,16 +36,24 @@
    #include "common.h"
 #endif
 
+class t_ThreadScan;
+
 class t_ThreadScanWorker: public QObject
 {
    Q_OBJECT
 
+   public:
+      t_ThreadScanWorker (t_ThreadScan *pThreadScan);
+
    public slots:
       virtual void SlotRescan (void) = 0;
 
    signals:
       void SignalScanFinished (t_pDeviceList);
-      void SignalScanStarted  (void)         ;
+      void SignalScanStarted  (void);
+
+   protected:
+      t_ThreadScan *poThreadScan;
 };
 
 class t_ThreadScanWorkerPartedLocal;
@@ -53,7 +62,7 @@ class t_ThreadScanWorkerParted: public t_ThreadScanWorker
    Q_OBJECT
 
    public:
-      t_ThreadScanWorkerParted (APIRET &rc);
+      t_ThreadScanWorkerParted (t_ThreadScan *pThreadScan, APIRET &rc);
      ~t_ThreadScanWorkerParted (void);
 
    public slots:
@@ -70,7 +79,7 @@ class t_ThreadScanWorkerHAL: public t_ThreadScanWorker
    Q_OBJECT
 
    public:
-      t_ThreadScanWorkerHAL (APIRET &rc);
+      t_ThreadScanWorkerHAL (t_ThreadScan *pThreadScan, APIRET &rc);
      ~t_ThreadScanWorkerHAL (void);
 
    private:
@@ -93,7 +102,7 @@ class t_ThreadScanWorkerDevKit: public t_ThreadScanWorker
    Q_OBJECT
 
    public:
-      t_ThreadScanWorkerDevKit (APIRET &rc);
+      t_ThreadScanWorkerDevKit (t_ThreadScan *pThreadScan, APIRET &rc);
      ~t_ThreadScanWorkerDevKit (void);
 
    private:
@@ -108,7 +117,7 @@ class t_ThreadScanWorkerDevKit: public t_ThreadScanWorker
 };
 
 
-class t_ThreadScan: public QThread
+class t_ThreadScan: public t_Thread
 {
    Q_OBJECT
 
