@@ -9,7 +9,7 @@
 //  Module:         Thread for writing data.
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -216,6 +216,9 @@ class t_OutputFileDD: public t_OutputFile
                LOG_ERROR ("[%s] Opening %s failed, errno=%d '%s'", QSTR_TO_PSZ (poDevice->LinuxDevice), QSTR_TO_PSZ (oFilename), errno, ToolErrorTranslateErrno (errno))
                return ERROR_THREADWRITE_OPEN_FAILED;
             }
+            if (poDevice->Acquisition.Clone) // The file size query from above would not need to be done in that
+               FileSize = poDevice->Size;    // case, however we do it anyway as it is a small access test as well.
+
             oRemainingFileSize = (t_uint64) FileSize;
             if (CONFIG(CalcImageFileMD5))
                CHK (HashMD5Init (&oCurrentHashContextMD5))
