@@ -9,7 +9,7 @@
 //  Module:         Thread for writing data.
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -29,7 +29,7 @@
 #ifndef __THREADWRITE_H__
 #define __THREADWRITE_H__
 
-#include <QThread>   //lint !e537 Repeated include
+#include "thread.h"
 
 #ifndef __DEVICE_H__
   #include "device.h"
@@ -38,7 +38,7 @@
 
 class t_ThreadWriteLocal;
 
-class t_ThreadWrite: public QThread
+class t_ThreadWrite: public t_Thread
 {
    Q_OBJECT
 
