@@ -109,6 +109,42 @@ unsigned int UtilGetMaxZcompressedBufferSize (unsigned int UncompressedBufferSiz
 }
 
 
+unsigned long long UtilGetInstalledRAM (void)
+{
+   unsigned long long Bytes;
+   int                Pages, PageSize;
+
+   Pages    = sysconf(_SC_PHYS_PAGES);
+   PageSize = sysconf(_SC_PAGESIZE);
+
+   if ((Pages < 0) || (PageSize < 0))
+   {
+      LOG_ERROR ("Amount of instlled RAM could not be determined, assume 256MB");
+      Bytes = 256*1024*1024;
+   }
+   else
+   {
+      Bytes = (unsigned long long) sysconf(_SC_PHYS_PAGES) * 
+              (unsigned long long) sysconf(_SC_PAGESIZE  );
+   }
+
+   return Bytes;
+}
+
+unsigned int UtilGetNumberOfCPUs (void)
+{
+   int ret;
+
+   ret = sysconf( _SC_NPROCESSORS_ONLN );
+   if (ret<0)
+   {
+      LOG_ERROR ("Number of CPUs could not be determined, assume 1 CPU");
+      ret = 1;
+   }
+   return (unsigned int) ret;
+}
+
+
 APIRET UtilInit (void)
 {   
    int MaxFifoBockSize;
