@@ -32,13 +32,15 @@
 void *UtilMemAlloc (size_t Size, const char *pFile, int Line);
 void  UtilMemFree  (void *pMem , const char *pFile, int Line);
 
+#define UTIL_MEM_ALLOC(Size) UtilMemAlloc (Size, __FILE__, __LINE__)
+#define UTIL_MEM_FREE(pMem)  UtilMemFree  (pMem, __FILE__, __LINE__)
+
 int          UtilCalcDecimals (t_uint64 Value);
 bool         UtilIsZero       (unsigned char *pData, unsigned int DataLen);
 unsigned int UtilGetMaxZcompressedBufferSize (unsigned int UncompressedBufferSize);
 
-#define UTIL_MEM_ALLOC(Size) UtilMemAlloc (Size, __FILE__, __LINE__)
-#define UTIL_MEM_FREE(pMem)  UtilMemFree  (pMem, __FILE__, __LINE__)
-
+unsigned long long UtilGetInstalledRAM (void); // Number of bytes
+unsigned int       UtilGetNumberOfCPUs (void);
 
 APIRET UtilInit   (void);
 APIRET UtilDeInit (void);
